import base64

from typing import Any

from google.protobuf.json_format import MessageToDict, ParseDict

from a2a.compat.v0_3 import types as types_v03
from a2a.compat.v0_3.versions import is_legacy_version
from a2a.types import a2a_pb2 as pb2_v10
from a2a.utils import constants, errors


_COMPAT_TO_CORE_TASK_STATE: dict[types_v03.TaskState, Any] = {
    types_v03.TaskState.unknown: pb2_v10.TaskState.TASK_STATE_UNSPECIFIED,
    types_v03.TaskState.submitted: pb2_v10.TaskState.TASK_STATE_SUBMITTED,
    types_v03.TaskState.working: pb2_v10.TaskState.TASK_STATE_WORKING,
    types_v03.TaskState.completed: pb2_v10.TaskState.TASK_STATE_COMPLETED,
    types_v03.TaskState.failed: pb2_v10.TaskState.TASK_STATE_FAILED,
    types_v03.TaskState.canceled: pb2_v10.TaskState.TASK_STATE_CANCELED,
    types_v03.TaskState.input_required: pb2_v10.TaskState.TASK_STATE_INPUT_REQUIRED,
    types_v03.TaskState.rejected: pb2_v10.TaskState.TASK_STATE_REJECTED,
    types_v03.TaskState.auth_required: pb2_v10.TaskState.TASK_STATE_AUTH_REQUIRED,
}

_CORE_TO_COMPAT_TASK_STATE: dict[Any, types_v03.TaskState] = {
    v: k for k, v in _COMPAT_TO_CORE_TASK_STATE.items()
}


def to_core_part(compat_part: types_v03.Part) -> pb2_v10.Part:  # noqa: PLR0912
    """Converts a v0.3 Part (Pydantic model) to a v1.0 core Part (Protobuf object)."""
    core_part = pb2_v10.Part()
    root = compat_part.root

    if isinstance(root, types_v03.TextPart):
        core_part.text = root.text
        if root.metadata is not None:
            ParseDict(root.metadata, core_part.metadata)

    elif isinstance(root, types_v03.DataPart):
        if root.metadata is None:
            data_part_compat = False
        else:
            meta = dict(root.metadata)
            data_part_compat = meta.pop('data_part_compat', False)
            if meta:
                ParseDict(meta, core_part.metadata)

        if data_part_compat:
            val = root.data['value']
            ParseDict(val, core_part.data)
        else:
            ParseDict(root.data, core_part.data.struct_value)

    elif isinstance(root, types_v03.FilePart):
        if isinstance(root.file, types_v03.FileWithBytes):
            core_part.raw = base64.b64decode(root.file.bytes)
            if root.file.mime_type:
                core_part.media_type = root.file.mime_type
            if root.file.name:
                core_part.filename = root.file.name
        elif isinstance(root.file, types_v03.FileWithUri):
            core_part.url = root.file.uri
            if root.file.mime_type:
                core_part.media_type = root.file.mime_type
            if root.file.name:
                core_part.filename = root.file.name

        if root.metadata is not None:
            ParseDict(root.metadata, core_part.metadata)

    return core_part


def to_compat_part(core_part: pb2_v10.Part) -> types_v03.Part:
    """Converts a v1.0 core Part (Protobuf object) to a v0.3 Part (Pydantic model)."""
    which = core_part.WhichOneof('content')
    metadata = (
        MessageToDict(core_part.metadata)
        if core_part.HasField('metadata')
        else None
    )

    if which == 'text':
        return types_v03.Part(
            root=types_v03.TextPart(text=core_part.text, metadata=metadata)
        )

    if which == 'data':
        # core_part.data is a google.protobuf.Value. It can be converted to dict.
        data_dict = MessageToDict(core_part.data)
        if not isinstance(data_dict, dict):
            data_dict = {'value': data_dict}
            metadata = metadata or {}
            metadata['data_part_compat'] = True

        return types_v03.Part(
            root=types_v03.DataPart(data=data_dict, metadata=metadata)
        )

    if which in ('raw', 'url'):
        media_type = core_part.media_type if core_part.media_type else None
        filename = core_part.filename if core_part.filename else None

        if which == 'raw':
            b64 = base64.b64encode(core_part.raw).decode('utf-8')
            file_obj_bytes = types_v03.FileWithBytes(
                bytes=b64, mime_type=media_type, name=filename
            )
            return types_v03.Part(
                root=types_v03.FilePart(file=file_obj_bytes, metadata=metadata)
            )
        file_obj_uri = types_v03.FileWithUri(
            uri=core_part.url, mime_type=media_type, name=filename
        )
        return types_v03.Part(
            root=types_v03.FilePart(file=file_obj_uri, metadata=metadata)
        )

    raise ValueError(f'Unknown part content type: {which}')


def to_core_message(compat_msg: types_v03.Message) -> pb2_v10.Message:
    """Convert message to v1.0 core type."""
    core_msg = pb2_v10.Message(
        message_id=compat_msg.message_id,
        context_id=compat_msg.context_id or '',
        task_id=compat_msg.task_id or '',
    )
    if compat_msg.reference_task_ids:
        core_msg.reference_task_ids.extend(compat_msg.reference_task_ids)

    if compat_msg.role == types_v03.Role.user:
        core_msg.role = pb2_v10.Role.ROLE_USER
    elif compat_msg.role == types_v03.Role.agent:
        core_msg.role = pb2_v10.Role.ROLE_AGENT

    if compat_msg.metadata:
        ParseDict(compat_msg.metadata, core_msg.metadata)

    if compat_msg.extensions:
        core_msg.extensions.extend(compat_msg.extensions)

    for p in compat_msg.parts:
        core_msg.parts.append(to_core_part(p))
    return core_msg


def to_compat_message(core_msg: pb2_v10.Message) -> types_v03.Message:
    """Convert message to v0.3 compat type."""
    role = (
        types_v03.Role.user
        if core_msg.role == pb2_v10.Role.ROLE_USER
        else types_v03.Role.agent
    )
    return types_v03.Message(
        message_id=core_msg.message_id,
        role=role,
        context_id=core_msg.context_id or None,
        task_id=core_msg.task_id or None,
        reference_task_ids=list(core_msg.reference_task_ids)
        if core_msg.reference_task_ids
        else None,
        metadata=MessageToDict(core_msg.metadata)
        if core_msg.metadata
        else None,
        extensions=list(core_msg.extensions) if core_msg.extensions else None,
        parts=[to_compat_part(p) for p in core_msg.parts],
    )


def to_core_task_status(
    compat_status: types_v03.TaskStatus,
) -> pb2_v10.TaskStatus:
    """Convert task status to v1.0 core type."""
    core_status = pb2_v10.TaskStatus()
    if compat_status.state:
        core_status.state = _COMPAT_TO_CORE_TASK_STATE.get(
            compat_status.state, pb2_v10.TaskState.TASK_STATE_UNSPECIFIED
        )

    if compat_status.message:
        core_status.message.CopyFrom(to_core_message(compat_status.message))
    if compat_status.timestamp:
        core_status.timestamp.FromJsonString(
            str(compat_status.timestamp).replace('+00:00', 'Z')
        )
    return core_status


def to_compat_task_status(
    core_status: pb2_v10.TaskStatus,
) -> types_v03.TaskStatus:
    """Convert task status to v0.3 compat type."""
    state_enum = _CORE_TO_COMPAT_TASK_STATE.get(
        core_status.state, types_v03.TaskState.unknown
    )

    update = (
        to_compat_message(core_status.message)
        if core_status.HasField('message')
        else None
    )
    ts = (
        core_status.timestamp.ToJsonString()
        if core_status.HasField('timestamp')
        else None
    )

    return types_v03.TaskStatus(state=state_enum, message=update, timestamp=ts)


def to_core_task(compat_task: types_v03.Task) -> pb2_v10.Task:
    """Convert task to v1.0 core type."""
    core_task = pb2_v10.Task(
        id=compat_task.id,
        context_id=compat_task.context_id,
    )
    if compat_task.status:
        core_task.status.CopyFrom(to_core_task_status(compat_task.status))
    if compat_task.history:
        for m in compat_task.history:
            core_task.history.append(to_core_message(m))
    if compat_task.artifacts:
        for a in compat_task.artifacts:
            core_task.artifacts.append(to_core_artifact(a))
    if compat_task.metadata:
        ParseDict(compat_task.metadata, core_task.metadata)
    return core_task


def to_compat_task(core_task: pb2_v10.Task) -> types_v03.Task:
    """Convert task to v0.3 compat type."""
    return types_v03.Task(
        id=core_task.id,
        context_id=core_task.context_id,
        status=to_compat_task_status(core_task.status)
        if core_task.HasField('status')
        else types_v03.TaskStatus(state=types_v03.TaskState.unknown),
        history=[to_compat_message(m) for m in core_task.history]
        if core_task.history
        else None,
        artifacts=[to_compat_artifact(a) for a in core_task.artifacts]
        if core_task.artifacts
        else None,
        metadata=MessageToDict(core_task.metadata)
        if core_task.HasField('metadata')
        else None,
    )


def to_core_authentication_info(
    compat_auth: types_v03.PushNotificationAuthenticationInfo,
) -> pb2_v10.AuthenticationInfo:
    """Convert authentication info to v1.0 core type."""
    core_auth = pb2_v10.AuthenticationInfo()
    if compat_auth.schemes:
        core_auth.scheme = compat_auth.schemes[0]
    if compat_auth.credentials:
        core_auth.credentials = compat_auth.credentials
    return core_auth


def to_compat_authentication_info(
    core_auth: pb2_v10.AuthenticationInfo,
) -> types_v03.PushNotificationAuthenticationInfo:
    """Convert authentication info to v0.3 compat type."""
    return types_v03.PushNotificationAuthenticationInfo(
        schemes=[core_auth.scheme] if core_auth.scheme else [],
        credentials=core_auth.credentials if core_auth.credentials else None,
    )


def to_core_push_notification_config(
    compat_config: types_v03.PushNotificationConfig,
) -> pb2_v10.TaskPushNotificationConfig:
    """Convert push notification config to v1.0 core type."""
    core_config = pb2_v10.TaskPushNotificationConfig(url=compat_config.url)
    if compat_config.id:
        core_config.id = compat_config.id
    if compat_config.token:
        core_config.token = compat_config.token
    if compat_config.authentication:
        core_config.authentication.CopyFrom(
            to_core_authentication_info(compat_config.authentication)
        )
    return core_config


def to_compat_push_notification_config(
    core_config: pb2_v10.TaskPushNotificationConfig,
) -> types_v03.PushNotificationConfig:
    """Convert push notification config to v0.3 compat type."""
    return types_v03.PushNotificationConfig(
        url=core_config.url if core_config.url else '',
        id=core_config.id if core_config.id else None,
        token=core_config.token if core_config.token else None,
        authentication=to_compat_authentication_info(core_config.authentication)
        if core_config.HasField('authentication')
        else None,
    )


def to_core_send_message_configuration(
    compat_config: types_v03.MessageSendConfiguration,
) -> pb2_v10.SendMessageConfiguration:
    """Convert send message configuration to v1.0 core type."""
    core_config = pb2_v10.SendMessageConfiguration()
    # Result will be blocking by default (return_immediately=False)
    if compat_config.accepted_output_modes:
        core_config.accepted_output_modes.extend(
            compat_config.accepted_output_modes
        )
    if compat_config.push_notification_config:
        core_config.task_push_notification_config.CopyFrom(
            to_core_push_notification_config(
                compat_config.push_notification_config
            )
        )
    if compat_config.history_length is not None:
        core_config.history_length = compat_config.history_length
    if compat_config.blocking is not None:
        core_config.return_immediately = not compat_config.blocking
    return core_config


def to_compat_send_message_configuration(
    core_config: pb2_v10.SendMessageConfiguration,
) -> types_v03.MessageSendConfiguration:
    """Convert send message configuration to v0.3 compat type."""
    return types_v03.MessageSendConfiguration(
        accepted_output_modes=list(core_config.accepted_output_modes)
        if core_config.accepted_output_modes
        else None,
        push_notification_config=to_compat_push_notification_config(
            core_config.task_push_notification_config
        )
        if core_config.HasField('task_push_notification_config')
        else None,
        history_length=core_config.history_length
        if core_config.HasField('history_length')
        else None,
        blocking=not core_config.return_immediately,
    )


def to_core_artifact(compat_artifact: types_v03.Artifact) -> pb2_v10.Artifact:
    """Convert artifact to v1.0 core type."""
    core_artifact = pb2_v10.Artifact(artifact_id=compat_artifact.artifact_id)
    if compat_artifact.name:
        core_artifact.name = compat_artifact.name
    if compat_artifact.description:
        core_artifact.description = compat_artifact.description
    for p in compat_artifact.parts:
        core_artifact.parts.append(to_core_part(p))
    if compat_artifact.metadata:
        ParseDict(compat_artifact.metadata, core_artifact.metadata)
    if compat_artifact.extensions:
        core_artifact.extensions.extend(compat_artifact.extensions)
    return core_artifact


def to_compat_artifact(core_artifact: pb2_v10.Artifact) -> types_v03.Artifact:
    """Convert artifact to v0.3 compat type."""
    return types_v03.Artifact(
        artifact_id=core_artifact.artifact_id,
        name=core_artifact.name if core_artifact.name else None,
        description=core_artifact.description
        if core_artifact.description
        else None,
        parts=[to_compat_part(p) for p in core_artifact.parts],
        metadata=MessageToDict(core_artifact.metadata)
        if core_artifact.HasField('metadata')
        else None,
        extensions=list(core_artifact.extensions)
        if core_artifact.extensions
        else None,
    )


def to_core_task_status_update_event(
    compat_event: types_v03.TaskStatusUpdateEvent,
) -> pb2_v10.TaskStatusUpdateEvent:
    """Convert task status update event to v1.0 core type."""
    core_event = pb2_v10.TaskStatusUpdateEvent(
        task_id=compat_event.task_id, context_id=compat_event.context_id
    )
    if compat_event.status:
        core_event.status.CopyFrom(to_core_task_status(compat_event.status))
    if compat_event.metadata:
        ParseDict(compat_event.metadata, core_event.metadata)
    return core_event


def to_compat_task_status_update_event(
    core_event: pb2_v10.TaskStatusUpdateEvent,
) -> types_v03.TaskStatusUpdateEvent:
    """Convert task status update event to v0.3 compat type."""
    status = (
        to_compat_task_status(core_event.status)
        if core_event.HasField('status')
        else types_v03.TaskStatus(state=types_v03.TaskState.unknown)
    )
    final = status.state in (
        types_v03.TaskState.completed,
        types_v03.TaskState.canceled,
        types_v03.TaskState.failed,
        types_v03.TaskState.rejected,
    )
    return types_v03.TaskStatusUpdateEvent(
        task_id=core_event.task_id,
        context_id=core_event.context_id,
        status=status,
        metadata=MessageToDict(core_event.metadata)
        if core_event.HasField('metadata')
        else None,
        final=final,
    )


def to_core_task_artifact_update_event(
    compat_event: types_v03.TaskArtifactUpdateEvent,
) -> pb2_v10.TaskArtifactUpdateEvent:
    """Convert task artifact update event to v1.0 core type."""
    core_event = pb2_v10.TaskArtifactUpdateEvent(
        task_id=compat_event.task_id, context_id=compat_event.context_id
    )
    if compat_event.artifact:
        core_event.artifact.CopyFrom(to_core_artifact(compat_event.artifact))
    if compat_event.append is not None:
        core_event.append = compat_event.append
    if compat_event.last_chunk is not None:
        core_event.last_chunk = compat_event.last_chunk
    if compat_event.metadata:
        ParseDict(compat_event.metadata, core_event.metadata)
    return core_event


def to_core_security_requirement(
    compat_req: dict[str, list[str]],
) -> pb2_v10.SecurityRequirement:
    """Convert security requirement to v1.0 core type."""
    core_req = pb2_v10.SecurityRequirement()
    for scheme_name, scopes in compat_req.items():
        sl = pb2_v10.StringList()
        sl.list.extend(scopes)
        core_req.schemes[scheme_name].CopyFrom(sl)
    return core_req


def to_compat_security_requirement(
    core_req: pb2_v10.SecurityRequirement,
) -> dict[str, list[str]]:
    """Convert security requirement to v0.3 compat type."""
    return {
        scheme_name: list(string_list.list)
        for scheme_name, string_list in core_req.schemes.items()
    }


def to_core_oauth_flows(
    compat_flows: types_v03.OAuthFlows,
) -> pb2_v10.OAuthFlows:
    """Convert oauth flows to v1.0 core type."""
    core_flows = pb2_v10.OAuthFlows()
    if compat_flows.authorization_code:
        f = pb2_v10.AuthorizationCodeOAuthFlow(
            authorization_url=compat_flows.authorization_code.authorization_url,
            token_url=compat_flows.authorization_code.token_url,
            scopes=compat_flows.authorization_code.scopes,
        )
        if compat_flows.authorization_code.refresh_url:
            f.refresh_url = compat_flows.authorization_code.refresh_url
        core_flows.authorization_code.CopyFrom(f)

    if compat_flows.client_credentials:
        f_client = pb2_v10.ClientCredentialsOAuthFlow(
            token_url=compat_flows.client_credentials.token_url,
            scopes=compat_flows.client_credentials.scopes,
        )
        if compat_flows.client_credentials.refresh_url:
            f_client.refresh_url = compat_flows.client_credentials.refresh_url
        core_flows.client_credentials.CopyFrom(f_client)

    if compat_flows.implicit:
        f_impl = pb2_v10.ImplicitOAuthFlow(
            authorization_url=compat_flows.implicit.authorization_url,
            scopes=compat_flows.implicit.scopes,
        )
        if compat_flows.implicit.refresh_url:
            f_impl.refresh_url = compat_flows.implicit.refresh_url
        core_flows.implicit.CopyFrom(f_impl)

    if compat_flows.password:
        f_pass = pb2_v10.PasswordOAuthFlow(
            token_url=compat_flows.password.token_url,
            scopes=compat_flows.password.scopes,
        )
        if compat_flows.password.refresh_url:
            f_pass.refresh_url = compat_flows.password.refresh_url
        core_flows.password.CopyFrom(f_pass)

    return core_flows


def to_compat_oauth_flows(
    core_flows: pb2_v10.OAuthFlows,
) -> types_v03.OAuthFlows:
    """Convert oauth flows to v0.3 compat type."""
    which = core_flows.WhichOneof('flow')
    auth_code, client_cred, implicit, password = None, None, None, None

    if which == 'authorization_code':
        auth_code = types_v03.AuthorizationCodeOAuthFlow(
            authorization_url=core_flows.authorization_code.authorization_url,
            token_url=core_flows.authorization_code.token_url,
            scopes=dict(core_flows.authorization_code.scopes),
            refresh_url=core_flows.authorization_code.refresh_url
            if core_flows.authorization_code.refresh_url
            else None,
        )
    elif which == 'client_credentials':
        client_cred = types_v03.ClientCredentialsOAuthFlow(
            token_url=core_flows.client_credentials.token_url,
            scopes=dict(core_flows.client_credentials.scopes),
            refresh_url=core_flows.client_credentials.refresh_url
            if core_flows.client_credentials.refresh_url
            else None,
        )
    elif which == 'implicit':
        implicit = types_v03.ImplicitOAuthFlow(
            authorization_url=core_flows.implicit.authorization_url,
            scopes=dict(core_flows.implicit.scopes),
            refresh_url=core_flows.implicit.refresh_url
            if core_flows.implicit.refresh_url
            else None,
        )
    elif which == 'password':
        password = types_v03.PasswordOAuthFlow(
            token_url=core_flows.password.token_url,
            scopes=dict(core_flows.password.scopes),
            refresh_url=core_flows.password.refresh_url
            if core_flows.password.refresh_url
            else None,
        )
    # Note: device_code from v1.0 is dropped since v0.3 doesn't support it

    return types_v03.OAuthFlows(
        authorization_code=auth_code,
        client_credentials=client_cred,
        implicit=implicit,
        password=password,
    )


def to_core_security_scheme(
    compat_scheme: types_v03.SecurityScheme,
) -> pb2_v10.SecurityScheme:
    """Convert security scheme to v1.0 core type."""
    core_scheme = pb2_v10.SecurityScheme()
    root = compat_scheme.root

    if isinstance(root, types_v03.APIKeySecurityScheme):
        core_scheme.api_key_security_scheme.location = root.in_.value
        core_scheme.api_key_security_scheme.name = root.name
        if root.description:
            core_scheme.api_key_security_scheme.description = root.description

    elif isinstance(root, types_v03.HTTPAuthSecurityScheme):
        core_scheme.http_auth_security_scheme.scheme = root.scheme
        if root.bearer_format:
            core_scheme.http_auth_security_scheme.bearer_format = (
                root.bearer_format
            )
        if root.description:
            core_scheme.http_auth_security_scheme.description = root.description

    elif isinstance(root, types_v03.OAuth2SecurityScheme):
        core_scheme.oauth2_security_scheme.flows.CopyFrom(
            to_core_oauth_flows(root.flows)
        )
        if root.oauth2_metadata_url:
            core_scheme.oauth2_security_scheme.oauth2_metadata_url = (
                root.oauth2_metadata_url
            )
        if root.description:
            core_scheme.oauth2_security_scheme.description = root.description

    elif isinstance(root, types_v03.OpenIdConnectSecurityScheme):
        core_scheme.open_id_connect_security_scheme.open_id_connect_url = (
            root.open_id_connect_url
        )
        if root.description:
            core_scheme.open_id_connect_security_scheme.description = (
                root.description
            )

    elif isinstance(root, types_v03.MutualTLSSecurityScheme):
        # Mutual TLS has no required fields other than description which is optional
        core_scheme.mtls_security_scheme.SetInParent()
        if root.description:
            core_scheme.mtls_security_scheme.description = root.description

    return core_scheme


def to_compat_security_scheme(
    core_scheme: pb2_v10.SecurityScheme,
) -> types_v03.SecurityScheme:
    """Convert security scheme to v0.3 compat type."""
    which = core_scheme.WhichOneof('scheme')

    if which == 'api_key_security_scheme':
        s_api = core_scheme.api_key_security_scheme
        return types_v03.SecurityScheme(
            root=types_v03.APIKeySecurityScheme(
                in_=types_v03.In(s_api.location),
                name=s_api.name,
                description=s_api.description if s_api.description else None,
            )
        )

    if which == 'http_auth_security_scheme':
        s_http = core_scheme.http_auth_security_scheme
        return types_v03.SecurityScheme(
            root=types_v03.HTTPAuthSecurityScheme(
                scheme=s_http.scheme,
                bearer_format=s_http.bearer_format
                if s_http.bearer_format
                else None,
                description=s_http.description if s_http.description else None,
            )
        )

    if which == 'oauth2_security_scheme':
        s_oauth = core_scheme.oauth2_security_scheme
        return types_v03.SecurityScheme(
            root=types_v03.OAuth2SecurityScheme(
                flows=to_compat_oauth_flows(s_oauth.flows),
                oauth2_metadata_url=s_oauth.oauth2_metadata_url
                if s_oauth.oauth2_metadata_url
                else None,
                description=s_oauth.description
                if s_oauth.description
                else None,
            )
        )

    if which == 'open_id_connect_security_scheme':
        s_oidc = core_scheme.open_id_connect_security_scheme
        return types_v03.SecurityScheme(
            root=types_v03.OpenIdConnectSecurityScheme(
                open_id_connect_url=s_oidc.open_id_connect_url,
                description=s_oidc.description if s_oidc.description else None,
            )
        )

    if which == 'mtls_security_scheme':
        s_mtls = core_scheme.mtls_security_scheme
        return types_v03.SecurityScheme(
            root=types_v03.MutualTLSSecurityScheme(
                description=s_mtls.description if s_mtls.description else None
            )
        )

    raise ValueError(f'Unknown security scheme type: {which}')


def to_core_agent_interface(
    compat_interface: types_v03.AgentInterface,
) -> pb2_v10.AgentInterface:
    """Convert agent interface to v1.0 core type."""
    return pb2_v10.AgentInterface(
        url=compat_interface.url,
        protocol_binding=compat_interface.transport,
        protocol_version=constants.PROTOCOL_VERSION_0_3,  # Defaulting for legacy
    )


def to_compat_agent_interface(
    core_interface: pb2_v10.AgentInterface,
) -> types_v03.AgentInterface:
    """Convert agent interface to v0.3 compat type."""
    return types_v03.AgentInterface(
        url=core_interface.url, transport=core_interface.protocol_binding
    )


def to_core_agent_provider(
    compat_provider: types_v03.AgentProvider,
) -> pb2_v10.AgentProvider:
    """Convert agent provider to v1.0 core type."""
    return pb2_v10.AgentProvider(
        url=compat_provider.url, organization=compat_provider.organization
    )


def to_compat_agent_provider(
    core_provider: pb2_v10.AgentProvider,
) -> types_v03.AgentProvider:
    """Convert agent provider to v0.3 compat type."""
    return types_v03.AgentProvider(
        url=core_provider.url, organization=core_provider.organization
    )


def to_core_agent_extension(
    compat_ext: types_v03.AgentExtension,
) -> pb2_v10.AgentExtension:
    """Convert agent extension to v1.0 core type."""
    core_ext = pb2_v10.AgentExtension()
    if compat_ext.uri:
        core_ext.uri = compat_ext.uri
    if compat_ext.description:
        core_ext.description = compat_ext.description
    if compat_ext.required is not None:
        core_ext.required = compat_ext.required
    if compat_ext.params:
        ParseDict(compat_ext.params, core_ext.params)
    return core_ext


def to_compat_agent_extension(
    core_ext: pb2_v10.AgentExtension,
) -> types_v03.AgentExtension:
    """Convert agent extension to v0.3 compat type."""
    return types_v03.AgentExtension(
        uri=core_ext.uri,
        description=core_ext.description if core_ext.description else None,
        required=core_ext.required,
        params=MessageToDict(core_ext.params)
        if core_ext.HasField('params')
        else None,
    )


def to_core_agent_capabilities(
    compat_cap: types_v03.AgentCapabilities,
) -> pb2_v10.AgentCapabilities:
    """Convert agent capabilities to v1.0 core type."""
    core_cap = pb2_v10.AgentCapabilities()
    if compat_cap.streaming is not None:
        core_cap.streaming = compat_cap.streaming
    if compat_cap.push_notifications is not None:
        core_cap.push_notifications = compat_cap.push_notifications
    if compat_cap.extensions:
        core_cap.extensions.extend(
            [to_core_agent_extension(e) for e in compat_cap.extensions]
        )
    return core_cap


def to_compat_agent_capabilities(
    core_cap: pb2_v10.AgentCapabilities,
) -> types_v03.AgentCapabilities:
    """Convert agent capabilities to v0.3 compat type."""
    return types_v03.AgentCapabilities(
        streaming=core_cap.streaming
        if core_cap.HasField('streaming')
        else None,
        push_notifications=core_cap.push_notifications
        if core_cap.HasField('push_notifications')
        else None,
        extensions=[to_compat_agent_extension(e) for e in core_cap.extensions]
        if core_cap.extensions
        else None,
        state_transition_history=None,  # No longer supported in v1.0
    )


def to_core_agent_skill(
    compat_skill: types_v03.AgentSkill,
) -> pb2_v10.AgentSkill:
    """Convert agent skill to v1.0 core type."""
    core_skill = pb2_v10.AgentSkill(
        id=compat_skill.id,
        name=compat_skill.name,
        description=compat_skill.description,
    )
    if compat_skill.tags:
        core_skill.tags.extend(compat_skill.tags)
    if compat_skill.examples:
        core_skill.examples.extend(compat_skill.examples)
    if compat_skill.input_modes:
        core_skill.input_modes.extend(compat_skill.input_modes)
    if compat_skill.output_modes:
        core_skill.output_modes.extend(compat_skill.output_modes)
    if compat_skill.security:
        core_skill.security_requirements.extend(
            [to_core_security_requirement(r) for r in compat_skill.security]
        )
    return core_skill


def to_compat_agent_skill(
    core_skill: pb2_v10.AgentSkill,
) -> types_v03.AgentSkill:
    """Convert agent skill to v0.3 compat type."""
    return types_v03.AgentSkill(
        id=core_skill.id,
        name=core_skill.name,
        description=core_skill.description,
        tags=list(core_skill.tags) if core_skill.tags else [],
        examples=list(core_skill.examples) if core_skill.examples else None,
        input_modes=list(core_skill.input_modes)
        if core_skill.input_modes
        else None,
        output_modes=list(core_skill.output_modes)
        if core_skill.output_modes
        else None,
        security=[
            to_compat_security_requirement(r)
            for r in core_skill.security_requirements
        ]
        if core_skill.security_requirements
        else None,
    )


def to_core_agent_card_signature(
    compat_sig: types_v03.AgentCardSignature,
) -> pb2_v10.AgentCardSignature:
    """Convert agent card signature to v1.0 core type."""
    core_sig = pb2_v10.AgentCardSignature(
        protected=compat_sig.protected, signature=compat_sig.signature
    )
    if compat_sig.header:
        ParseDict(compat_sig.header, core_sig.header)
    return core_sig


def to_compat_agent_card_signature(
    core_sig: pb2_v10.AgentCardSignature,
) -> types_v03.AgentCardSignature:
    """Convert agent card signature to v0.3 compat type."""
    return types_v03.AgentCardSignature(
        protected=core_sig.protected,
        signature=core_sig.signature,
        header=MessageToDict(core_sig.header)
        if core_sig.HasField('header')
        else None,
    )


def to_core_agent_card(compat_card: types_v03.AgentCard) -> pb2_v10.AgentCard:
    """Convert agent card to v1.0 core type."""
    core_card = pb2_v10.AgentCard(
        name=compat_card.name,
        description=compat_card.description,
        version=compat_card.version,
    )

    # Map primary interface
    primary_interface = pb2_v10.AgentInterface(
        url=compat_card.url,
        protocol_binding=compat_card.preferred_transport or 'JSONRPC',
        protocol_version=compat_card.protocol_version
        or constants.PROTOCOL_VERSION_0_3,
    )
    core_card.supported_interfaces.append(primary_interface)

    if compat_card.additional_interfaces:
        core_card.supported_interfaces.extend(
            [
                to_core_agent_interface(i)
                for i in compat_card.additional_interfaces
            ]
        )

    if compat_card.provider:
        core_card.provider.CopyFrom(
            to_core_agent_provider(compat_card.provider)
        )

    if compat_card.documentation_url:
        core_card.documentation_url = compat_card.documentation_url

    if compat_card.icon_url:
        core_card.icon_url = compat_card.icon_url

    core_cap = to_core_agent_capabilities(compat_card.capabilities)
    if compat_card.supports_authenticated_extended_card is not None:
        core_cap.extended_agent_card = (
            compat_card.supports_authenticated_extended_card
        )
    core_card.capabilities.CopyFrom(core_cap)

    if compat_card.security_schemes:
        for k, v in compat_card.security_schemes.items():
            core_card.security_schemes[k].CopyFrom(to_core_security_scheme(v))

    if compat_card.security:
        core_card.security_requirements.extend(
            [to_core_security_requirement(r) for r in compat_card.security]
        )

    if compat_card.default_input_modes:
        core_card.default_input_modes.extend(compat_card.default_input_modes)

    if compat_card.default_output_modes:
        core_card.default_output_modes.extend(compat_card.default_output_modes)

    if compat_card.skills:
        core_card.skills.extend(
            [to_core_agent_skill(s) for s in compat_card.skills]
        )

    if compat_card.signatures:
        core_card.signatures.extend(
            [to_core_agent_card_signature(s) for s in compat_card.signatures]
        )

    return core_card


def to_compat_agent_card(core_card: pb2_v10.AgentCard) -> types_v03.AgentCard:
    # Map supported interfaces back to legacy layout
    """Convert agent card to v0.3 compat type."""
    compat_interfaces = [
        interface
        for interface in core_card.supported_interfaces
        if (
            (not interface.protocol_version)
            or is_legacy_version(interface.protocol_version)
        )
    ]
    if not compat_interfaces:
        raise errors.VersionNotSupportedError(
            'AgentCard must have at least one interface with compatible protocol version.'
        )

    primary_interface = compat_interfaces[0]
    additional_interfaces = [
        to_compat_agent_interface(i) for i in compat_interfaces[1:]
    ]

    compat_cap = to_compat_agent_capabilities(core_card.capabilities)
    supports_authenticated_extended_card = (
        core_card.capabilities.extended_agent_card
        if core_card.capabilities.HasField('extended_agent_card')
        else None
    )

    return types_v03.AgentCard(
        name=core_card.name,
        description=core_card.description,
        version=core_card.version,
        url=primary_interface.url,
        preferred_transport=primary_interface.protocol_binding,
        protocol_version=primary_interface.protocol_version
        or constants.PROTOCOL_VERSION_0_3,
        additional_interfaces=additional_interfaces or None,
        provider=to_compat_agent_provider(core_card.provider)
        if core_card.HasField('provider')
        else None,
        documentation_url=core_card.documentation_url
        if core_card.HasField('documentation_url')
        else None,
        icon_url=core_card.icon_url if core_card.HasField('icon_url') else None,
        capabilities=compat_cap,
        supports_authenticated_extended_card=supports_authenticated_extended_card,
        security_schemes={
            k: to_compat_security_scheme(v)
            for k, v in core_card.security_schemes.items()
        }
        if core_card.security_schemes
        else None,
        security=[
            to_compat_security_requirement(r)
            for r in core_card.security_requirements
        ]
        if core_card.security_requirements
        else None,
        default_input_modes=list(core_card.default_input_modes)
        if core_card.default_input_modes
        else [],
        default_output_modes=list(core_card.default_output_modes)
        if core_card.default_output_modes
        else [],
        skills=[to_compat_agent_skill(s) for s in core_card.skills]
        if core_card.skills
        else [],
        signatures=[
            to_compat_agent_card_signature(s) for s in core_card.signatures
        ]
        if core_card.signatures
        else None,
    )


def to_compat_task_artifact_update_event(
    core_event: pb2_v10.TaskArtifactUpdateEvent,
) -> types_v03.TaskArtifactUpdateEvent:
    """Convert task artifact update event to v0.3 compat type."""
    return types_v03.TaskArtifactUpdateEvent(
        task_id=core_event.task_id,
        context_id=core_event.context_id,
        artifact=to_compat_artifact(core_event.artifact),
        append=core_event.append,
        last_chunk=core_event.last_chunk,
        metadata=MessageToDict(core_event.metadata)
        if core_event.HasField('metadata')
        else None,
    )


def to_core_task_push_notification_config(
    compat_config: types_v03.TaskPushNotificationConfig,
) -> pb2_v10.TaskPushNotificationConfig:
    """Convert task push notification config to v1.0 core type."""
    core_config = pb2_v10.TaskPushNotificationConfig(
        task_id=compat_config.task_id
    )
    if compat_config.push_notification_config:
        core_config.MergeFrom(
            to_core_push_notification_config(
                compat_config.push_notification_config
            )
        )
    return core_config


def to_compat_task_push_notification_config(
    core_config: pb2_v10.TaskPushNotificationConfig,
) -> types_v03.TaskPushNotificationConfig:
    """Convert task push notification config to v0.3 compat type."""
    return types_v03.TaskPushNotificationConfig(
        task_id=core_config.task_id,
        push_notification_config=to_compat_push_notification_config(
            core_config
        ),
    )


def to_core_send_message_request(
    compat_req: types_v03.SendMessageRequest,
) -> pb2_v10.SendMessageRequest:
    """Convert send message request to v1.0 core type."""
    core_req = pb2_v10.SendMessageRequest()
    if compat_req.params.message:
        core_req.message.CopyFrom(to_core_message(compat_req.params.message))
    if compat_req.params.configuration:
        core_req.configuration.CopyFrom(
            to_core_send_message_configuration(compat_req.params.configuration)
        )
    if compat_req.params.metadata:
        ParseDict(compat_req.params.metadata, core_req.metadata)
    return core_req


def to_compat_send_message_request(
    core_req: pb2_v10.SendMessageRequest, request_id: str | int
) -> types_v03.SendMessageRequest:
    """Convert send message request to v0.3 compat type."""
    return types_v03.SendMessageRequest(
        id=request_id,
        params=types_v03.MessageSendParams(
            message=to_compat_message(core_req.message),
            configuration=to_compat_send_message_configuration(
                core_req.configuration
            )
            if core_req.HasField('configuration')
            else None,
            metadata=MessageToDict(core_req.metadata)
            if core_req.HasField('metadata')
            else None,
        ),
    )


def to_core_get_task_request(
    compat_req: types_v03.GetTaskRequest,
) -> pb2_v10.GetTaskRequest:
    """Convert get task request to v1.0 core type."""
    core_req = pb2_v10.GetTaskRequest()
    core_req.id = compat_req.params.id
    if compat_req.params.history_length is not None:
        core_req.history_length = compat_req.params.history_length
    return core_req


def to_compat_get_task_request(
    core_req: pb2_v10.GetTaskRequest, request_id: str | int
) -> types_v03.GetTaskRequest:
    """Convert get task request to v0.3 compat type."""
    return types_v03.GetTaskRequest(
        id=request_id,
        params=types_v03.TaskQueryParams(
            id=core_req.id,
            history_length=core_req.history_length
            if core_req.HasField('history_length')
            else None,
        ),
    )


def to_core_cancel_task_request(
    compat_req: types_v03.CancelTaskRequest,
) -> pb2_v10.CancelTaskRequest:
    """Convert cancel task request to v1.0 core type."""
    core_req = pb2_v10.CancelTaskRequest(id=compat_req.params.id)
    if compat_req.params.metadata:
        ParseDict(compat_req.params.metadata, core_req.metadata)
    return core_req


def to_compat_cancel_task_request(
    core_req: pb2_v10.CancelTaskRequest, request_id: str | int
) -> types_v03.CancelTaskRequest:
    """Convert cancel task request to v0.3 compat type."""
    return types_v03.CancelTaskRequest(
        id=request_id,
        params=types_v03.TaskIdParams(
            id=core_req.id,
            metadata=MessageToDict(core_req.metadata)
            if core_req.HasField('metadata')
            else None,
        ),
    )


def to_core_get_task_push_notification_config_request(
    compat_req: types_v03.GetTaskPushNotificationConfigRequest,
) -> pb2_v10.GetTaskPushNotificationConfigRequest:
    """Convert get task push notification config request to v1.0 core type."""
    if isinstance(
        compat_req.params, types_v03.GetTaskPushNotificationConfigParams
    ):
        return pb2_v10.GetTaskPushNotificationConfigRequest(
            task_id=compat_req.params.id,
            id=compat_req.params.push_notification_config_id,
        )
    return pb2_v10.GetTaskPushNotificationConfigRequest(
        task_id=compat_req.params.id
    )


def to_compat_get_task_push_notification_config_request(
    core_req: pb2_v10.GetTaskPushNotificationConfigRequest,
    request_id: str | int,
) -> types_v03.GetTaskPushNotificationConfigRequest:
    """Convert get task push notification config request to v0.3 compat type."""
    params: (
        types_v03.GetTaskPushNotificationConfigParams | types_v03.TaskIdParams
    )
    if core_req.id:
        params = types_v03.GetTaskPushNotificationConfigParams(
            id=core_req.task_id, push_notification_config_id=core_req.id
        )
    else:
        params = types_v03.TaskIdParams(id=core_req.task_id)
    return types_v03.GetTaskPushNotificationConfigRequest(
        id=request_id, params=params
    )


def to_core_delete_task_push_notification_config_request(
    compat_req: types_v03.DeleteTaskPushNotificationConfigRequest,
) -> pb2_v10.DeleteTaskPushNotificationConfigRequest:
    """Convert delete task push notification config request to v1.0 core type."""
    return pb2_v10.DeleteTaskPushNotificationConfigRequest(
        task_id=compat_req.params.id,
        id=compat_req.params.push_notification_config_id,
    )


def to_compat_delete_task_push_notification_config_request(
    core_req: pb2_v10.DeleteTaskPushNotificationConfigRequest,
    request_id: str | int,
) -> types_v03.DeleteTaskPushNotificationConfigRequest:
    """Convert delete task push notification config request to v0.3 compat type."""
    return types_v03.DeleteTaskPushNotificationConfigRequest(
        id=request_id,
        params=types_v03.DeleteTaskPushNotificationConfigParams(
            id=core_req.task_id, push_notification_config_id=core_req.id
        ),
    )


def to_core_create_task_push_notification_config_request(
    compat_req: types_v03.SetTaskPushNotificationConfigRequest,
) -> pb2_v10.TaskPushNotificationConfig:
    """Convert create task push notification config request to v1.0 core type."""
    core_req = pb2_v10.TaskPushNotificationConfig(
        task_id=compat_req.params.task_id
    )
    if compat_req.params.push_notification_config:
        core_req.MergeFrom(
            to_core_push_notification_config(
                compat_req.params.push_notification_config
            )
        )
    return core_req


def to_compat_create_task_push_notification_config_request(
    core_req: pb2_v10.TaskPushNotificationConfig,
    request_id: str | int,
) -> types_v03.SetTaskPushNotificationConfigRequest:
    """Convert create task push notification config request to v0.3 compat type."""
    return types_v03.SetTaskPushNotificationConfigRequest(
        id=request_id,
        params=types_v03.TaskPushNotificationConfig(
            task_id=core_req.task_id,
            push_notification_config=to_compat_push_notification_config(
                core_req
            ),
        ),
    )


def to_core_subscribe_to_task_request(
    compat_req: types_v03.TaskResubscriptionRequest,
) -> pb2_v10.SubscribeToTaskRequest:
    """Convert subscribe to task request to v1.0 core type."""
    return pb2_v10.SubscribeToTaskRequest(id=compat_req.params.id)


def to_compat_subscribe_to_task_request(
    core_req: pb2_v10.SubscribeToTaskRequest, request_id: str | int
) -> types_v03.TaskResubscriptionRequest:
    """Convert subscribe to task request to v0.3 compat type."""
    return types_v03.TaskResubscriptionRequest(
        id=request_id, params=types_v03.TaskIdParams(id=core_req.id)
    )


def to_core_list_task_push_notification_config_request(
    compat_req: types_v03.ListTaskPushNotificationConfigRequest,
) -> pb2_v10.ListTaskPushNotificationConfigsRequest:
    """Convert list task push notification config request to v1.0 core type."""
    core_req = pb2_v10.ListTaskPushNotificationConfigsRequest()
    if compat_req.params.id:
        core_req.task_id = compat_req.params.id
    return core_req


def to_compat_list_task_push_notification_config_request(
    core_req: pb2_v10.ListTaskPushNotificationConfigsRequest,
    request_id: str | int,
) -> types_v03.ListTaskPushNotificationConfigRequest:
    """Convert list task push notification config request to v0.3 compat type."""
    return types_v03.ListTaskPushNotificationConfigRequest(
        id=request_id,
        params=types_v03.ListTaskPushNotificationConfigParams(
            id=core_req.task_id
        ),
    )


def to_core_list_task_push_notification_config_response(
    compat_res: types_v03.ListTaskPushNotificationConfigResponse,
) -> pb2_v10.ListTaskPushNotificationConfigsResponse:
    """Convert list task push notification config response to v1.0 core type."""
    core_res = pb2_v10.ListTaskPushNotificationConfigsResponse()
    root = compat_res.root
    if isinstance(
        root, types_v03.ListTaskPushNotificationConfigSuccessResponse
    ):
        for c in root.result:
            core_res.configs.append(to_core_task_push_notification_config(c))
    return core_res


def to_compat_list_task_push_notification_config_response(
    core_res: pb2_v10.ListTaskPushNotificationConfigsResponse,
    request_id: str | int | None = None,
) -> types_v03.ListTaskPushNotificationConfigResponse:
    """Convert list task push notification config response to v0.3 compat type."""
    return types_v03.ListTaskPushNotificationConfigResponse(
        root=types_v03.ListTaskPushNotificationConfigSuccessResponse(
            id=request_id,
            result=[
                to_compat_task_push_notification_config(c)
                for c in core_res.configs
            ],
        )
    )


def to_core_send_message_response(
    compat_res: types_v03.SendMessageResponse,
) -> pb2_v10.SendMessageResponse:
    """Convert send message response to v1.0 core type."""
    core_res = pb2_v10.SendMessageResponse()
    root = compat_res.root
    if isinstance(root, types_v03.SendMessageSuccessResponse):
        if isinstance(root.result, types_v03.Task):
            core_res.task.CopyFrom(to_core_task(root.result))
        else:
            core_res.message.CopyFrom(to_core_message(root.result))
    return core_res


def to_compat_send_message_response(
    core_res: pb2_v10.SendMessageResponse, request_id: str | int | None = None
) -> types_v03.SendMessageResponse:
    """Convert send message response to v0.3 compat type."""
    if core_res.HasField('task'):
        result_task = to_compat_task(core_res.task)
        return types_v03.SendMessageResponse(
            root=types_v03.SendMessageSuccessResponse(
                id=request_id, result=result_task
            )
        )
    result_msg = to_compat_message(core_res.message)
    return types_v03.SendMessageResponse(
        root=types_v03.SendMessageSuccessResponse(
            id=request_id, result=result_msg
        )
    )


def to_core_stream_response(
    compat_res: types_v03.SendStreamingMessageSuccessResponse,
) -> pb2_v10.StreamResponse:
    """Convert stream response to v1.0 core type."""
    core_res = pb2_v10.StreamResponse()
    root = compat_res.result

    if isinstance(root, types_v03.Message):
        core_res.message.CopyFrom(to_core_message(root))
    elif isinstance(root, types_v03.Task):
        core_res.task.CopyFrom(to_core_task(root))
    elif isinstance(root, types_v03.TaskStatusUpdateEvent):
        core_res.status_update.CopyFrom(to_core_task_status_update_event(root))
    elif isinstance(root, types_v03.TaskArtifactUpdateEvent):
        core_res.artifact_update.CopyFrom(
            to_core_task_artifact_update_event(root)
        )

    return core_res


def to_compat_stream_response(
    core_res: pb2_v10.StreamResponse, request_id: str | int | None = None
) -> types_v03.SendStreamingMessageSuccessResponse:
    """Convert stream response to v0.3 compat type."""
    which = core_res.WhichOneof('payload')
    if which == 'message':
        return types_v03.SendStreamingMessageSuccessResponse(
            id=request_id, result=to_compat_message(core_res.message)
        )
    if which == 'task':
        return types_v03.SendStreamingMessageSuccessResponse(
            id=request_id, result=to_compat_task(core_res.task)
        )
    if which == 'status_update':
        return types_v03.SendStreamingMessageSuccessResponse(
            id=request_id,
            result=to_compat_task_status_update_event(core_res.status_update),
        )
    if which == 'artifact_update':
        return types_v03.SendStreamingMessageSuccessResponse(
            id=request_id,
            result=to_compat_task_artifact_update_event(
                core_res.artifact_update
            ),
        )

    raise ValueError(f'Unknown stream response event type: {which}')


def to_core_get_extended_agent_card_request(
    compat_req: types_v03.GetAuthenticatedExtendedCardRequest,
) -> pb2_v10.GetExtendedAgentCardRequest:
    """Convert get extended agent card request to v1.0 core type."""
    return pb2_v10.GetExtendedAgentCardRequest()


def to_compat_get_extended_agent_card_request(
    core_req: pb2_v10.GetExtendedAgentCardRequest, request_id: str | int
) -> types_v03.GetAuthenticatedExtendedCardRequest:
    """Convert get extended agent card request to v0.3 compat type."""
    return types_v03.GetAuthenticatedExtendedCardRequest(id=request_id)



"""Shared header name constants for v0.3 extension compatibility.

The current spec uses ``A2A-Extensions``. v0.3 used the ``X-`` prefixed
``X-A2A-Extensions`` form. v0.3 compat servers and clients accept/emit both
names so they can interoperate with peers that only know the legacy one.
"""

from a2a.client.service_parameters import ServiceParameters
from a2a.extensions.common import HTTP_EXTENSION_HEADER


LEGACY_HTTP_EXTENSION_HEADER = f'X-{HTTP_EXTENSION_HEADER}'


def add_legacy_extension_header(parameters: ServiceParameters) -> None:
    """Mirrors the ``A2A-Extensions`` parameter under its legacy name in-place.

    Used by v0.3 compat client transports so that requests can be understood
    by older v0.3 servers that only recognize ``X-A2A-Extensions``.
    """
    if (
        HTTP_EXTENSION_HEADER in parameters
        and LEGACY_HTTP_EXTENSION_HEADER not in parameters
    ):
        parameters[LEGACY_HTTP_EXTENSION_HEADER] = parameters[
            HTTP_EXTENSION_HEADER
        ]



"""Context builders that add v0.3 backwards-compatibility for extensions.

The current spec uses ``A2A-Extensions`` (RFC 6648, no ``X-`` prefix). v0.3
clients still send the old ``X-A2A-Extensions`` name, so the v0.3 compat
adapters wrap the default builders with these classes to recognize both names.
"""

from typing import TYPE_CHECKING

from a2a.compat.v0_3.extension_headers import LEGACY_HTTP_EXTENSION_HEADER
from a2a.extensions.common import get_requested_extensions
from a2a.server.context import ServerCallContext


if TYPE_CHECKING:
    import grpc

    from starlette.requests import Request

    from a2a.server.request_handlers.grpc_handler import (
        GrpcServerCallContextBuilder,
    )
    from a2a.server.routes.common import ServerCallContextBuilder


def _get_legacy_grpc_extensions(
    context: 'grpc.aio.ServicerContext',
) -> list[str]:
    md = context.invocation_metadata()
    if md is None:
        return []
    lower_key = LEGACY_HTTP_EXTENSION_HEADER.lower()
    return [
        e if isinstance(e, str) else e.decode('utf-8')
        for k, e in md
        if k.lower() == lower_key
    ]


class V03ServerCallContextBuilder:
    """Wraps a ServerCallContextBuilder to also accept the legacy header.

    Recognizes the v0.3 ``X-A2A-Extensions`` HTTP header in addition to the
    spec ``A2A-Extensions``.
    """

    def __init__(self, inner: 'ServerCallContextBuilder') -> None:
        self._inner = inner

    def build(self, request: 'Request') -> ServerCallContext:
        """Builds a ServerCallContext, merging legacy extension headers."""
        context = self._inner.build(request)
        context.requested_extensions |= get_requested_extensions(
            request.headers.getlist(LEGACY_HTTP_EXTENSION_HEADER)
        )
        return context


class V03GrpcServerCallContextBuilder:
    """Wraps a GrpcServerCallContextBuilder to also accept the legacy metadata.

    Recognizes the v0.3 ``X-A2A-Extensions`` gRPC metadata key in addition to
    the spec ``A2A-Extensions``.
    """

    def __init__(self, inner: 'GrpcServerCallContextBuilder') -> None:
        self._inner = inner

    def build(self, context: 'grpc.aio.ServicerContext') -> ServerCallContext:
        """Builds a ServerCallContext, merging legacy extension metadata."""
        server_context = self._inner.build(context)
        server_context.requested_extensions |= get_requested_extensions(
            _get_legacy_grpc_extensions(context)
        )
        return server_context



import asyncio
import logging
import os

from logging.config import fileConfig

from sqlalchemy import Connection, pool
from sqlalchemy.ext.asyncio import async_engine_from_config

from a2a.server.models import Base

try:
    from alembic import context
except ImportError as e:
    raise ImportError(
        "Migrations require Alembic. Install with: 'pip install a2a-sdk[db-cli]'."
    ) from e


# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Mandatory database configuration
db_url = os.getenv('DATABASE_URL')
if not db_url:
    raise RuntimeError(
        'DATABASE_URL environment variable is not set. '
        "Please set it (e.g., export DATABASE_URL='sqlite+aiosqlite:///./my-database.db') before running migrations "
        'or use the --database-url flag.'
    )
config.set_main_option('sqlalchemy.url', db_url)

# Interpret the config file for Python logging.
# This line sets up loggers basically.
if (
    config.config_file_name is not None
    and os.path.exists(config.config_file_name)
    and config.config_file_name.endswith('.ini')
):
    fileConfig(config.config_file_name)

if config.get_main_option('verbose') == 'true':
    logging.getLogger('sqlalchemy.engine').setLevel(logging.INFO)

# add your model's MetaData object here for 'autogenerate' support
target_metadata = Base.metadata

# Version table name to avoid conflicts with existing alembic_version tables in the database.
# This ensures that the migration history for A2A is tracked separately.
VERSION_TABLE = 'a2a_alembic_version'


def run_migrations_offline() -> None:
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = config.get_main_option('sqlalchemy.url')
    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={'paramstyle': 'named'},
        version_table=VERSION_TABLE,
    )

    with context.begin_transaction():
        context.run_migrations()


def do_run_migrations(connection: Connection) -> None:
    """Run migrations in 'online' mode.

    This function is called within a synchronous context (via run_sync)
    to configure the migration context with the provided connection
    and target metadata, then execute the migrations within a transaction.

    Args:
        connection: The SQLAlchemy connection to use for the migrations.
    """
    context.configure(
        connection=connection,
        target_metadata=target_metadata,
        version_table=VERSION_TABLE,
    )

    with context.begin_transaction():
        context.run_migrations()


async def run_async_migrations() -> None:
    """Run migrations using an Engine.

    In this scenario we need to create an Engine
    and associate a connection with the context.
    """
    connectable = async_engine_from_config(
        config.get_section(config.config_ini_section, {}),
        prefix='sqlalchemy.',
        poolclass=pool.NullPool,
    )

    async with connectable.connect() as connection:
        await connection.run_sync(do_run_migrations)

    await connectable.dispose()


def run_migrations_online() -> None:
    """Run migrations in 'online' mode."""
    asyncio.run(run_async_migrations())


if context.is_offline_mode():
    logging.info('Running migrations in offline mode.')
    run_migrations_offline()
else:
    logging.info('Running migrations in online mode.')
    run_migrations_online()



#!/usr/bin/env python3
"""Smoke test for installations of a2a-sdk with various extras.

This script verifies that the public API modules associated with a
given installation profile can be imported without pulling in modules
that belong to other (uninstalled) optional extras.

It is designed to run WITHOUT pytest or any dev dependencies -- just
a clean venv with `pip install a2a-sdk[<profile>]`.

Usage:
    python scripts/test_install_smoke.py [profile]

    profile defaults to "base" and selects which set of modules to
    smoke-test. Available profiles:
      base        -- `pip install a2a-sdk`
      http-server -- `pip install a2a-sdk[http-server]`
      grpc        -- `pip install a2a-sdk[grpc]`
      telemetry   -- `pip install a2a-sdk[telemetry]`
      sql         -- `pip install a2a-sdk[sql]`

Exit codes:
    0 - All imports for the profile succeeded
    1 - One or more imports failed
"""

from __future__ import annotations

import importlib
import sys


# Core modules that MUST be importable with only base dependencies.
# These are the public API surface that every user gets with
# `pip install a2a-sdk` (no extras).
#
# Do NOT add modules here that require optional extras (grpc,
# http-server, sql, signing, telemetry, vertex, etc.).
# Those modules are expected to fail without their extras installed
# and should use try/except ImportError guards internally.
CORE_MODULES = [
    'a2a',
    'a2a.client',
    'a2a.client.auth',
    'a2a.client.base_client',
    'a2a.client.card_resolver',
    'a2a.client.client',
    'a2a.client.client_factory',
    'a2a.client.errors',
    'a2a.client.interceptors',
    'a2a.client.optionals',
    'a2a.client.transports',
    'a2a.server',
    'a2a.server.agent_execution',
    'a2a.server.context',
    'a2a.server.events',
    'a2a.server.request_handlers',
    'a2a.server.tasks',
    'a2a.types',
    'a2a.utils',
    'a2a.utils.constants',
    'a2a.utils.error_handlers',
    'a2a.utils.version_validator',
    'a2a.utils.proto_utils',
    'a2a.utils.task',
    'a2a.helpers.agent_card',
    'a2a.helpers.proto_helpers',
]

# Modules that MUST be importable with only the base + `http-server`
# extras installed (no `grpc`, `sql`, `signing`, `telemetry`, etc.).
#
# A user building a Starlette/FastAPI A2A server with
# `pip install a2a-sdk[http-server]` should be able to import these
# without the gRPC stack being present on the system.
HTTP_SERVER_MODULES = [
    'a2a.server.routes',
    'a2a.server.routes.agent_card_routes',
    'a2a.server.routes.common',
    'a2a.server.routes.jsonrpc_dispatcher',
    'a2a.server.routes.jsonrpc_routes',
    'a2a.server.routes.rest_dispatcher',
    'a2a.server.routes.rest_routes',
]

# Modules that MUST be importable with only the base + `grpc` extras
# installed (no `http-server`, `sql`, `signing`, `telemetry`, etc.).
GRPC_MODULES = [
    'a2a.server.request_handlers.grpc_handler',
    'a2a.client.transports.grpc',
    'a2a.compat.v0_3.grpc_handler',
    'a2a.compat.v0_3.grpc_transport',
]

# Modules that MUST be importable with only the base + `telemetry`
# extras installed.
TELEMETRY_MODULES = [
    'a2a.utils.telemetry',
]

# Modules that MUST be importable with only the base + `sql` extras
# installed (covers postgresql/mysql/sqlite drivers via SQLAlchemy).
SQL_MODULES = [
    'a2a.server.models',
    'a2a.server.tasks.database_task_store',
    'a2a.server.tasks.database_push_notification_config_store',
]


PROFILES: dict[str, list[str]] = {
    'base': CORE_MODULES,
    'http-server': CORE_MODULES + HTTP_SERVER_MODULES,
    'grpc': CORE_MODULES + GRPC_MODULES,
    'telemetry': CORE_MODULES + TELEMETRY_MODULES,
    'sql': CORE_MODULES + SQL_MODULES,
}


def main() -> int:
    profile = sys.argv[1] if len(sys.argv) > 1 else 'base'
    if profile not in PROFILES:
        print(f'Unknown profile {profile!r}. Available: {sorted(PROFILES)}')
        return 1

    modules = PROFILES[profile]
    failures: list[str] = []
    successes: list[str] = []

    for module_name in modules:
        try:
            importlib.import_module(module_name)
            successes.append(module_name)
        except Exception as e:  # noqa: BLE001, PERF203
            failures.append(f'{module_name}: {e}')

    print(f'Profile: {profile}')
    print(f'Tested {len(modules)} modules')
    print(f'  Passed: {len(successes)}')
    print(f'  Failed: {len(failures)}')

    if failures:
        print('\nFAILED imports:')
        for failure in failures:
            print(f'  - {failure}')
        return 1

    print('\nAll modules imported successfully.')
    return 0


if __name__ == '__main__':
    sys.exit(main())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: a2a.proto
# Protobuf Python Version: 5.29.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    3,
    '',
    'a2a.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.api import client_pb2 as google_dot_api_dot_client__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\ta2a.proto\x12\tlf.a2a.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x17google/api/client.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xa6\x02\n\x18SendMessageConfiguration\x12\x32\n\x15\x61\x63\x63\x65pted_output_modes\x18\x01 \x03(\tR\x13\x61\x63\x63\x65ptedOutputModes\x12h\n\x1dtask_push_notification_config\x18\x02 \x01(\x0b\x32%.lf.a2a.v1.TaskPushNotificationConfigR\x1ataskPushNotificationConfig\x12*\n\x0ehistory_length\x18\x03 \x01(\x05H\x00R\rhistoryLength\x88\x01\x01\x12-\n\x12return_immediately\x18\x04 \x01(\x08R\x11returnImmediatelyB\x11\n\x0f_history_length\"\x84\x02\n\x04Task\x12\x13\n\x02id\x18\x01 \x01(\tB\x03\xe0\x41\x02R\x02id\x12\x1d\n\ncontext_id\x18\x02 \x01(\tR\tcontextId\x12\x32\n\x06status\x18\x03 \x01(\x0b\x32\x15.lf.a2a.v1.TaskStatusB\x03\xe0\x41\x02R\x06status\x12\x31\n\tartifacts\x18\x04 \x03(\x0b\x32\x13.lf.a2a.v1.ArtifactR\tartifacts\x12,\n\x07history\x18\x05 \x03(\x0b\x32\x12.lf.a2a.v1.MessageR\x07history\x12\x33\n\x08metadata\x18\x06 \x01(\x0b\x32\x17.google.protobuf.StructR\x08metadata\"\xa5\x01\n\nTaskStatus\x12/\n\x05state\x18\x01 \x01(\x0e\x32\x14.lf.a2a.v1.TaskStateB\x03\xe0\x41\x02R\x05state\x12,\n\x07message\x18\x02 \x01(\x0b\x32\x12.lf.a2a.v1.MessageR\x07message\x12\x38\n\ttimestamp\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\"\xed\x01\n\x04Part\x12\x14\n\x04text\x18\x01 \x01(\tH\x00R\x04text\x12\x12\n\x03raw\x18\x02 \x01(\x0cH\x00R\x03raw\x12\x12\n\x03url\x18\x03 \x01(\tH\x00R\x03url\x12,\n\x04\x64\x61ta\x18\x04 \x01(\x0b\x32\x16.google.protobuf.ValueH\x00R\x04\x64\x61ta\x12\x33\n\x08metadata\x18\x05 \x01(\x0b\x32\x17.google.protobuf.StructR\x08metadata\x12\x1a\n\x08\x66ilename\x18\x06 \x01(\tR\x08\x66ilename\x12\x1d\n\nmedia_type\x18\x07 \x01(\tR\tmediaTypeB\t\n\x07\x63ontent\"\xbe\x02\n\x07Message\x12\"\n\nmessage_id\x18\x01 \x01(\tB\x03\xe0\x41\x02R\tmessageId\x12\x1d\n\ncontext_id\x18\x02 \x01(\tR\tcontextId\x12\x17\n\x07task_id\x18\x03 \x01(\tR\x06taskId\x12(\n\x04role\x18\x04 \x01(\x0e\x32\x0f.lf.a2a.v1.RoleB\x03\xe0\x41\x02R\x04role\x12*\n\x05parts\x18\x05 \x03(\x0b\x32\x0f.lf.a2a.v1.PartB\x03\xe0\x41\x02R\x05parts\x12\x33\n\x08metadata\x18\x06 \x01(\x0b\x32\x17.google.protobuf.StructR\x08metadata\x12\x1e\n\nextensions\x18\x07 \x03(\tR\nextensions\x12,\n\x12reference_task_ids\x18\x08 \x03(\tR\x10referenceTaskIds\"\xe7\x01\n\x08\x41rtifact\x12$\n\x0b\x61rtifact_id\x18\x01 \x01(\tB\x03\xe0\x41\x02R\nartifactId\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12 \n\x0b\x64\x65scription\x18\x03 \x01(\tR\x0b\x64\x65scription\x12*\n\x05parts\x18\x04 \x03(\x0b\x32\x0f.lf.a2a.v1.PartB\x03\xe0\x41\x02R\x05parts\x12\x33\n\x08metadata\x18\x05 \x01(\x0b\x32\x17.google.protobuf.StructR\x08metadata\x12\x1e\n\nextensions\x18\x06 \x03(\tR\nextensions\"\xc2\x01\n\x15TaskStatusUpdateEvent\x12\x1c\n\x07task_id\x18\x01 \x01(\tB\x03\xe0\x41\x02R\x06taskId\x12\"\n\ncontext_id\x18\x02 \x01(\tB\x03\xe0\x41\x02R\tcontextId\x12\x32\n\x06status\x18\x03 \x01(\x0b\x32\x15.lf.a2a.v1.TaskStatusB\x03\xe0\x41\x02R\x06status\x12\x33\n\x08metadata\x18\x04 \x01(\x0b\x32\x17.google.protobuf.StructR\x08metadata\"\xfd\x01\n\x17TaskArtifactUpdateEvent\x12\x1c\n\x07task_id\x18\x01 \x01(\tB\x03\xe0\x41\x02R\x06taskId\x12\"\n\ncontext_id\x18\x02 \x01(\tB\x03\xe0\x41\x02R\tcontextId\x12\x34\n\x08\x61rtifact\x18\x03 \x01(\x0b\x32\x13.lf.a2a.v1.ArtifactB\x03\xe0\x41\x02R\x08\x61rtifact\x12\x16\n\x06\x61ppend\x18\x04 \x01(\x08R\x06\x61ppend\x12\x1d\n\nlast_chunk\x18\x05 \x01(\x08R\tlastChunk\x12\x33\n\x08metadata\x18\x06 \x01(\x0b\x32\x17.google.protobuf.StructR\x08metadata\"S\n\x12\x41uthenticationInfo\x12\x1b\n\x06scheme\x18\x01 \x01(\tB\x03\xe0\x41\x02R\x06scheme\x12 \n\x0b\x63redentials\x18\x02 \x01(\tR\x0b\x63redentials\"\x9f\x01\n\x0e\x41gentInterface\x12\x15\n\x03url\x18\x01 \x01(\tB\x03\xe0\x41\x02R\x03url\x12.\n\x10protocol_binding\x18\x02 \x01(\tB\x03\xe0\x41\x02R\x0fprotocolBinding\x12\x16\n\x06tenant\x18\x03 \x01(\tR\x06tenant\x12.\n\x10protocol_version\x18\x04 \x01(\tB\x03\xe0\x41\x02R\x0fprotocolVersion\"\x98\x07\n\tAgentCard\x12\x17\n\x04name\x18\x01 \x01(\tB\x03\xe0\x41\x02R\x04name\x12%\n\x0b\x64\x65scription\x18\x02 \x01(\tB\x03\xe0\x41\x02R\x0b\x64\x65scription\x12Q\n\x14supported_interfaces\x18\x03 \x03(\x0b\x32\x19.lf.a2a.v1.AgentInterfaceB\x03\xe0\x41\x02R\x13supportedInterfaces\x12\x34\n\x08provider\x18\x04 \x01(\x0b\x32\x18.lf.a2a.v1.AgentProviderR\x08provider\x12\x1d\n\x07version\x18\x05 \x01(\tB\x03\xe0\x41\x02R\x07version\x12\x30\n\x11\x64ocumentation_url\x18\x06 \x01(\tH\x00R\x10\x64ocumentationUrl\x88\x01\x01\x12\x45\n\x0c\x63\x61pabilities\x18\x07 \x01(\x0b\x32\x1c.lf.a2a.v1.AgentCapabilitiesB\x03\xe0\x41\x02R\x0c\x63\x61pabilities\x12T\n\x10security_schemes\x18\x08 \x03(\x0b\x32).lf.a2a.v1.AgentCard.SecuritySchemesEntryR\x0fsecuritySchemes\x12S\n\x15security_requirements\x18\t \x03(\x0b\x32\x1e.lf.a2a.v1.SecurityRequirementR\x14securityRequirements\x12\x33\n\x13\x64\x65\x66\x61ult_input_modes\x18\n \x03(\tB\x03\xe0\x41\x02R\x11\x64\x65\x66\x61ultInputModes\x12\x35\n\x14\x64\x65\x66\x61ult_output_modes\x18\x0b \x03(\tB\x03\xe0\x41\x02R\x12\x64\x65\x66\x61ultOutputModes\x12\x32\n\x06skills\x18\x0c \x03(\x0b\x32\x15.lf.a2a.v1.AgentSkillB\x03\xe0\x41\x02R\x06skills\x12=\n\nsignatures\x18\r \x03(\x0b\x32\x1d.lf.a2a.v1.AgentCardSignatureR\nsignatures\x12\x1e\n\x08icon_url\x18\x0e \x01(\tH\x01R\x07iconUrl\x88\x01\x01\x1a]\n\x14SecuritySchemesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12/\n\x05value\x18\x02 \x01(\x0b\x32\x19.lf.a2a.v1.SecuritySchemeR\x05value:\x02\x38\x01\x42\x14\n\x12_documentation_urlB\x0b\n\t_icon_url\"O\n\rAgentProvider\x12\x15\n\x03url\x18\x01 \x01(\tB\x03\xe0\x41\x02R\x03url\x12\'\n\x0corganization\x18\x02 \x01(\tB\x03\xe0\x41\x02R\x0corganization\"\x97\x02\n\x11\x41gentCapabilities\x12!\n\tstreaming\x18\x01 \x01(\x08H\x00R\tstreaming\x88\x01\x01\x12\x32\n\x12push_notifications\x18\x02 \x01(\x08H\x01R\x11pushNotifications\x88\x01\x01\x12\x39\n\nextensions\x18\x03 \x03(\x0b\x32\x19.lf.a2a.v1.AgentExtensionR\nextensions\x12\x33\n\x13\x65xtended_agent_card\x18\x04 \x01(\x08H\x02R\x11\x65xtendedAgentCard\x88\x01\x01\x42\x0c\n\n_streamingB\x15\n\x13_push_notificationsB\x16\n\x14_extended_agent_card\"\x91\x01\n\x0e\x41gentExtension\x12\x10\n\x03uri\x18\x01 \x01(\tR\x03uri\x12 \n\x0b\x64\x65scription\x18\x02 \x01(\tR\x0b\x64\x65scription\x12\x1a\n\x08required\x18\x03 \x01(\x08R\x08required\x12/\n\x06params\x18\x04 \x01(\x0b\x32\x17.google.protobuf.StructR\x06params\"\xaf\x02\n\nAgentSkill\x12\x13\n\x02id\x18\x01 \x01(\tB\x03\xe0\x41\x02R\x02id\x12\x17\n\x04name\x18\x02 \x01(\tB\x03\xe0\x41\x02R\x04name\x12%\n\x0b\x64\x65scription\x18\x03 \x01(\tB\x03\xe0\x41\x02R\x0b\x64\x65scription\x12\x17\n\x04tags\x18\x04 \x03(\tB\x03\xe0\x41\x02R\x04tags\x12\x1a\n\x08\x65xamples\x18\x05 \x03(\tR\x08\x65xamples\x12\x1f\n\x0binput_modes\x18\x06 \x03(\tR\ninputModes\x12!\n\x0coutput_modes\x18\x07 \x03(\tR\x0boutputModes\x12S\n\x15security_requirements\x18\x08 \x03(\x0b\x32\x1e.lf.a2a.v1.SecurityRequirementR\x14securityRequirements\"\x8b\x01\n\x12\x41gentCardSignature\x12!\n\tprotected\x18\x01 \x01(\tB\x03\xe0\x41\x02R\tprotected\x12!\n\tsignature\x18\x02 \x01(\tB\x03\xe0\x41\x02R\tsignature\x12/\n\x06header\x18\x03 \x01(\x0b\x32\x17.google.protobuf.StructR\x06header\"\xd1\x01\n\x1aTaskPushNotificationConfig\x12\x16\n\x06tenant\x18\x01 \x01(\tR\x06tenant\x12\x0e\n\x02id\x18\x02 \x01(\tR\x02id\x12\x17\n\x07task_id\x18\x03 \x01(\tR\x06taskId\x12\x15\n\x03url\x18\x04 \x01(\tB\x03\xe0\x41\x02R\x03url\x12\x14\n\x05token\x18\x05 \x01(\tR\x05token\x12\x45\n\x0e\x61uthentication\x18\x06 \x01(\x0b\x32\x1d.lf.a2a.v1.AuthenticationInfoR\x0e\x61uthentication\" \n\nStringList\x12\x12\n\x04list\x18\x01 \x03(\tR\x04list\"\xaf\x01\n\x13SecurityRequirement\x12\x45\n\x07schemes\x18\x01 \x03(\x0b\x32+.lf.a2a.v1.SecurityRequirement.SchemesEntryR\x07schemes\x1aQ\n\x0cSchemesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12+\n\x05value\x18\x02 \x01(\x0b\x32\x15.lf.a2a.v1.StringListR\x05value:\x02\x38\x01\"\xf5\x03\n\x0eSecurityScheme\x12X\n\x17\x61pi_key_security_scheme\x18\x01 \x01(\x0b\x32\x1f.lf.a2a.v1.APIKeySecuritySchemeH\x00R\x14\x61piKeySecurityScheme\x12^\n\x19http_auth_security_scheme\x18\x02 \x01(\x0b\x32!.lf.a2a.v1.HTTPAuthSecuritySchemeH\x00R\x16httpAuthSecurityScheme\x12W\n\x16oauth2_security_scheme\x18\x03 \x01(\x0b\x32\x1f.lf.a2a.v1.OAuth2SecuritySchemeH\x00R\x14oauth2SecurityScheme\x12n\n\x1fopen_id_connect_security_scheme\x18\x04 \x01(\x0b\x32&.lf.a2a.v1.OpenIdConnectSecuritySchemeH\x00R\x1bopenIdConnectSecurityScheme\x12V\n\x14mtls_security_scheme\x18\x05 \x01(\x0b\x32\".lf.a2a.v1.MutualTlsSecuritySchemeH\x00R\x12mtlsSecuritySchemeB\x08\n\x06scheme\"r\n\x14\x41PIKeySecurityScheme\x12 \n\x0b\x64\x65scription\x18\x01 \x01(\tR\x0b\x64\x65scription\x12\x1f\n\x08location\x18\x02 \x01(\tB\x03\xe0\x41\x02R\x08location\x12\x17\n\x04name\x18\x03 \x01(\tB\x03\xe0\x41\x02R\x04name\"|\n\x16HTTPAuthSecurityScheme\x12 \n\x0b\x64\x65scription\x18\x01 \x01(\tR\x0b\x64\x65scription\x12\x1b\n\x06scheme\x18\x02 \x01(\tB\x03\xe0\x41\x02R\x06scheme\x12#\n\rbearer_format\x18\x03 \x01(\tR\x0c\x62\x65\x61rerFormat\"\x9a\x01\n\x14OAuth2SecurityScheme\x12 \n\x0b\x64\x65scription\x18\x01 \x01(\tR\x0b\x64\x65scription\x12\x30\n\x05\x66lows\x18\x02 \x01(\x0b\x32\x15.lf.a2a.v1.OAuthFlowsB\x03\xe0\x41\x02R\x05\x66lows\x12.\n\x13oauth2_metadata_url\x18\x03 \x01(\tR\x11oauth2MetadataUrl\"s\n\x1bOpenIdConnectSecurityScheme\x12 \n\x0b\x64\x65scription\x18\x01 \x01(\tR\x0b\x64\x65scription\x12\x32\n\x13open_id_connect_url\x18\x02 \x01(\tB\x03\xe0\x41\x02R\x10openIdConnectUrl\";\n\x17MutualTlsSecurityScheme\x12 \n\x0b\x64\x65scription\x18\x01 \x01(\tR\x0b\x64\x65scription\"\x87\x03\n\nOAuthFlows\x12V\n\x12\x61uthorization_code\x18\x01 \x01(\x0b\x32%.lf.a2a.v1.AuthorizationCodeOAuthFlowH\x00R\x11\x61uthorizationCode\x12V\n\x12\x63lient_credentials\x18\x02 \x01(\x0b\x32%.lf.a2a.v1.ClientCredentialsOAuthFlowH\x00R\x11\x63lientCredentials\x12>\n\x08implicit\x18\x03 \x01(\x0b\x32\x1c.lf.a2a.v1.ImplicitOAuthFlowB\x02\x18\x01H\x00R\x08implicit\x12>\n\x08password\x18\x04 \x01(\x0b\x32\x1c.lf.a2a.v1.PasswordOAuthFlowB\x02\x18\x01H\x00R\x08password\x12\x41\n\x0b\x64\x65vice_code\x18\x05 \x01(\x0b\x32\x1e.lf.a2a.v1.DeviceCodeOAuthFlowH\x00R\ndeviceCodeB\x06\n\x04\x66low\"\xc1\x02\n\x1a\x41uthorizationCodeOAuthFlow\x12\x30\n\x11\x61uthorization_url\x18\x01 \x01(\tB\x03\xe0\x41\x02R\x10\x61uthorizationUrl\x12 \n\ttoken_url\x18\x02 \x01(\tB\x03\xe0\x41\x02R\x08tokenUrl\x12\x1f\n\x0brefresh_url\x18\x03 \x01(\tR\nrefreshUrl\x12N\n\x06scopes\x18\x04 \x03(\x0b\x32\x31.lf.a2a.v1.AuthorizationCodeOAuthFlow.ScopesEntryB\x03\xe0\x41\x02R\x06scopes\x12#\n\rpkce_required\x18\x05 \x01(\x08R\x0cpkceRequired\x1a\x39\n\x0bScopesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xea\x01\n\x1a\x43lientCredentialsOAuthFlow\x12 \n\ttoken_url\x18\x01 \x01(\tB\x03\xe0\x41\x02R\x08tokenUrl\x12\x1f\n\x0brefresh_url\x18\x02 \x01(\tR\nrefreshUrl\x12N\n\x06scopes\x18\x03 \x03(\x0b\x32\x31.lf.a2a.v1.ClientCredentialsOAuthFlow.ScopesEntryB\x03\xe0\x41\x02R\x06scopes\x1a\x39\n\x0bScopesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xde\x01\n\x11ImplicitOAuthFlow\x12+\n\x11\x61uthorization_url\x18\x01 \x01(\tR\x10\x61uthorizationUrl\x12\x1f\n\x0brefresh_url\x18\x02 \x01(\tR\nrefreshUrl\x12@\n\x06scopes\x18\x03 \x03(\x0b\x32(.lf.a2a.v1.ImplicitOAuthFlow.ScopesEntryR\x06scopes\x1a\x39\n\x0bScopesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xce\x01\n\x11PasswordOAuthFlow\x12\x1b\n\ttoken_url\x18\x01 \x01(\tR\x08tokenUrl\x12\x1f\n\x0brefresh_url\x18\x02 \x01(\tR\nrefreshUrl\x12@\n\x06scopes\x18\x03 \x03(\x0b\x32(.lf.a2a.v1.PasswordOAuthFlow.ScopesEntryR\x06scopes\x1a\x39\n\x0bScopesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\x9b\x02\n\x13\x44\x65viceCodeOAuthFlow\x12=\n\x18\x64\x65vice_authorization_url\x18\x01 \x01(\tB\x03\xe0\x41\x02R\x16\x64\x65viceAuthorizationUrl\x12 \n\ttoken_url\x18\x02 \x01(\tB\x03\xe0\x41\x02R\x08tokenUrl\x12\x1f\n\x0brefresh_url\x18\x03 \x01(\tR\nrefreshUrl\x12G\n\x06scopes\x18\x04 \x03(\x0b\x32*.lf.a2a.v1.DeviceCodeOAuthFlow.ScopesEntryB\x03\xe0\x41\x02R\x06scopes\x1a\x39\n\x0bScopesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xdf\x01\n\x12SendMessageRequest\x12\x16\n\x06tenant\x18\x01 \x01(\tR\x06tenant\x12\x31\n\x07message\x18\x02 \x01(\x0b\x32\x12.lf.a2a.v1.MessageB\x03\xe0\x41\x02R\x07message\x12I\n\rconfiguration\x18\x03 \x01(\x0b\x32#.lf.a2a.v1.SendMessageConfigurationR\rconfiguration\x12\x33\n\x08metadata\x18\x04 \x01(\x0b\x32\x17.google.protobuf.StructR\x08metadata\"|\n\x0eGetTaskRequest\x12\x16\n\x06tenant\x18\x01 \x01(\tR\x06tenant\x12\x13\n\x02id\x18\x02 \x01(\tB\x03\xe0\x41\x02R\x02id\x12*\n\x0ehistory_length\x18\x03 \x01(\x05H\x00R\rhistoryLength\x88\x01\x01\x42\x11\n\x0f_history_length\"\x9f\x03\n\x10ListTasksRequest\x12\x16\n\x06tenant\x18\x01 \x01(\tR\x06tenant\x12\x1d\n\ncontext_id\x18\x02 \x01(\tR\tcontextId\x12,\n\x06status\x18\x03 \x01(\x0e\x32\x14.lf.a2a.v1.TaskStateR\x06status\x12 \n\tpage_size\x18\x04 \x01(\x05H\x00R\x08pageSize\x88\x01\x01\x12\x1d\n\npage_token\x18\x05 \x01(\tR\tpageToken\x12*\n\x0ehistory_length\x18\x06 \x01(\x05H\x01R\rhistoryLength\x88\x01\x01\x12P\n\x16status_timestamp_after\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x14statusTimestampAfter\x12\x30\n\x11include_artifacts\x18\x08 \x01(\x08H\x02R\x10includeArtifacts\x88\x01\x01\x42\x0c\n\n_page_sizeB\x11\n\x0f_history_lengthB\x14\n\x12_include_artifacts\"\xb2\x01\n\x11ListTasksResponse\x12*\n\x05tasks\x18\x01 \x03(\x0b\x32\x0f.lf.a2a.v1.TaskB\x03\xe0\x41\x02R\x05tasks\x12+\n\x0fnext_page_token\x18\x02 \x01(\tB\x03\xe0\x41\x02R\rnextPageToken\x12 \n\tpage_size\x18\x03 \x01(\x05\x42\x03\xe0\x41\x02R\x08pageSize\x12\"\n\ntotal_size\x18\x04 \x01(\x05\x42\x03\xe0\x41\x02R\ttotalSize\"u\n\x11\x43\x61ncelTaskRequest\x12\x16\n\x06tenant\x18\x01 \x01(\tR\x06tenant\x12\x13\n\x02id\x18\x02 \x01(\tB\x03\xe0\x41\x02R\x02id\x12\x33\n\x08metadata\x18\x03 \x01(\x0b\x32\x17.google.protobuf.StructR\x08metadata\"q\n$GetTaskPushNotificationConfigRequest\x12\x16\n\x06tenant\x18\x01 \x01(\tR\x06tenant\x12\x1c\n\x07task_id\x18\x02 \x01(\tB\x03\xe0\x41\x02R\x06taskId\x12\x13\n\x02id\x18\x03 \x01(\tB\x03\xe0\x41\x02R\x02id\"t\n\'DeleteTaskPushNotificationConfigRequest\x12\x16\n\x06tenant\x18\x01 \x01(\tR\x06tenant\x12\x1c\n\x07task_id\x18\x02 \x01(\tB\x03\xe0\x41\x02R\x06taskId\x12\x13\n\x02id\x18\x03 \x01(\tB\x03\xe0\x41\x02R\x02id\"E\n\x16SubscribeToTaskRequest\x12\x16\n\x06tenant\x18\x01 \x01(\tR\x06tenant\x12\x13\n\x02id\x18\x02 \x01(\tB\x03\xe0\x41\x02R\x02id\"\x9a\x01\n&ListTaskPushNotificationConfigsRequest\x12\x16\n\x06tenant\x18\x04 \x01(\tR\x06tenant\x12\x1c\n\x07task_id\x18\x01 \x01(\tB\x03\xe0\x41\x02R\x06taskId\x12\x1b\n\tpage_size\x18\x02 \x01(\x05R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"5\n\x1bGetExtendedAgentCardRequest\x12\x16\n\x06tenant\x18\x01 \x01(\tR\x06tenant\"w\n\x13SendMessageResponse\x12%\n\x04task\x18\x01 \x01(\x0b\x32\x0f.lf.a2a.v1.TaskH\x00R\x04task\x12.\n\x07message\x18\x02 \x01(\x0b\x32\x12.lf.a2a.v1.MessageH\x00R\x07messageB\t\n\x07payload\"\x8a\x02\n\x0eStreamResponse\x12%\n\x04task\x18\x01 \x01(\x0b\x32\x0f.lf.a2a.v1.TaskH\x00R\x04task\x12.\n\x07message\x18\x02 \x01(\x0b\x32\x12.lf.a2a.v1.MessageH\x00R\x07message\x12G\n\rstatus_update\x18\x03 \x01(\x0b\x32 .lf.a2a.v1.TaskStatusUpdateEventH\x00R\x0cstatusUpdate\x12M\n\x0f\x61rtifact_update\x18\x04 \x01(\x0b\x32\".lf.a2a.v1.TaskArtifactUpdateEventH\x00R\x0e\x61rtifactUpdateB\t\n\x07payload\"\x92\x01\n\'ListTaskPushNotificationConfigsResponse\x12?\n\x07\x63onfigs\x18\x01 \x03(\x0b\x32%.lf.a2a.v1.TaskPushNotificationConfigR\x07\x63onfigs\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken*\xf9\x01\n\tTaskState\x12\x1a\n\x16TASK_STATE_UNSPECIFIED\x10\x00\x12\x18\n\x14TASK_STATE_SUBMITTED\x10\x01\x12\x16\n\x12TASK_STATE_WORKING\x10\x02\x12\x18\n\x14TASK_STATE_COMPLETED\x10\x03\x12\x15\n\x11TASK_STATE_FAILED\x10\x04\x12\x17\n\x13TASK_STATE_CANCELED\x10\x05\x12\x1d\n\x19TASK_STATE_INPUT_REQUIRED\x10\x06\x12\x17\n\x13TASK_STATE_REJECTED\x10\x07\x12\x1c\n\x18TASK_STATE_AUTH_REQUIRED\x10\x08*;\n\x04Role\x12\x14\n\x10ROLE_UNSPECIFIED\x10\x00\x12\r\n\tROLE_USER\x10\x01\x12\x0e\n\nROLE_AGENT\x10\x02\x32\x97\x0f\n\nA2AService\x12\x83\x01\n\x0bSendMessage\x12\x1d.lf.a2a.v1.SendMessageRequest\x1a\x1e.lf.a2a.v1.SendMessageResponse\"5\x82\xd3\xe4\x93\x02/\"\r/message:send:\x01*Z\x1b\"\x16/{tenant}/message:send:\x01*\x12\x8d\x01\n\x14SendStreamingMessage\x12\x1d.lf.a2a.v1.SendMessageRequest\x1a\x19.lf.a2a.v1.StreamResponse\"9\x82\xd3\xe4\x93\x02\x33\"\x0f/message:stream:\x01*Z\x1d\"\x18/{tenant}/message:stream:\x01*0\x01\x12k\n\x07GetTask\x12\x19.lf.a2a.v1.GetTaskRequest\x1a\x0f.lf.a2a.v1.Task\"4\xda\x41\x02id\x82\xd3\xe4\x93\x02)\x12\r/tasks/{id=*}Z\x18\x12\x16/{tenant}/tasks/{id=*}\x12i\n\tListTasks\x12\x1b.lf.a2a.v1.ListTasksRequest\x1a\x1c.lf.a2a.v1.ListTasksResponse\"!\x82\xd3\xe4\x93\x02\x1b\x12\x06/tasksZ\x11\x12\x0f/{tenant}/tasks\x12\x80\x01\n\nCancelTask\x12\x1c.lf.a2a.v1.CancelTaskRequest\x1a\x0f.lf.a2a.v1.Task\"C\x82\xd3\xe4\x93\x02=\"\x14/tasks/{id=*}:cancel:\x01*Z\"\"\x1d/{tenant}/tasks/{id=*}:cancel:\x01*\x12\x96\x01\n\x0fSubscribeToTask\x12!.lf.a2a.v1.SubscribeToTaskRequest\x1a\x19.lf.a2a.v1.StreamResponse\"C\x82\xd3\xe4\x93\x02=\x12\x17/tasks/{id=*}:subscribeZ\"\x12 /{tenant}/tasks/{id=*}:subscribe0\x01\x12\xf3\x01\n CreateTaskPushNotificationConfig\x12%.lf.a2a.v1.TaskPushNotificationConfig\x1a%.lf.a2a.v1.TaskPushNotificationConfig\"\x80\x01\xda\x41\x0etask_id,config\x82\xd3\xe4\x93\x02i\"*/tasks/{task_id=*}/pushNotificationConfigs:\x01*Z8\"3/{tenant}/tasks/{task_id=*}/pushNotificationConfigs:\x01*\x12\xfe\x01\n\x1dGetTaskPushNotificationConfig\x12/.lf.a2a.v1.GetTaskPushNotificationConfigRequest\x1a%.lf.a2a.v1.TaskPushNotificationConfig\"\x84\x01\xda\x41\ntask_id,id\x82\xd3\xe4\x93\x02q\x12\x31/tasks/{task_id=*}/pushNotificationConfigs/{id=*}Z<\x12:/{tenant}/tasks/{task_id=*}/pushNotificationConfigs/{id=*}\x12\xfd\x01\n\x1fListTaskPushNotificationConfigs\x12\x31.lf.a2a.v1.ListTaskPushNotificationConfigsRequest\x1a\x32.lf.a2a.v1.ListTaskPushNotificationConfigsResponse\"s\xda\x41\x07task_id\x82\xd3\xe4\x93\x02\x63\x12*/tasks/{task_id=*}/pushNotificationConfigsZ5\x12\x33/{tenant}/tasks/{task_id=*}/pushNotificationConfigs\x12\x8f\x01\n\x14GetExtendedAgentCard\x12&.lf.a2a.v1.GetExtendedAgentCardRequest\x1a\x14.lf.a2a.v1.AgentCard\"9\x82\xd3\xe4\x93\x02\x33\x12\x12/extendedAgentCardZ\x1d\x12\x1b/{tenant}/extendedAgentCard\x12\xf5\x01\n DeleteTaskPushNotificationConfig\x12\x32.lf.a2a.v1.DeleteTaskPushNotificationConfigRequest\x1a\x16.google.protobuf.Empty\"\x84\x01\xda\x41\ntask_id,id\x82\xd3\xe4\x93\x02q*1/tasks/{task_id=*}/pushNotificationConfigs/{id=*}Z<*:/{tenant}/tasks/{task_id=*}/pushNotificationConfigs/{id=*}B|\n\rcom.lf.a2a.v1B\x08\x41\x32\x61ProtoP\x01Z\x1bgoogle.golang.org/lf/a2a/v1\xa2\x02\x03LAX\xaa\x02\tLf.A2a.V1\xca\x02\tLf\\A2a\\V1\xe2\x02\x15Lf\\A2a\\V1\\GPBMetadata\xea\x02\x0bLf::A2a::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'a2a_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\rcom.lf.a2a.v1B\010A2aProtoP\001Z\033google.golang.org/lf/a2a/v1\242\002\003LAX\252\002\tLf.A2a.V1\312\002\tLf\\A2a\\V1\342\002\025Lf\\A2a\\V1\\GPBMetadata\352\002\013Lf::A2a::V1'
  _globals['_TASK'].fields_by_name['id']._loaded_options = None
  _globals['_TASK'].fields_by_name['id']._serialized_options = b'\340A\002'
  _globals['_TASK'].fields_by_name['status']._loaded_options = None
  _globals['_TASK'].fields_by_name['status']._serialized_options = b'\340A\002'
  _globals['_TASKSTATUS'].fields_by_name['state']._loaded_options = None
  _globals['_TASKSTATUS'].fields_by_name['state']._serialized_options = b'\340A\002'
  _globals['_MESSAGE'].fields_by_name['message_id']._loaded_options = None
  _globals['_MESSAGE'].fields_by_name['message_id']._serialized_options = b'\340A\002'
  _globals['_MESSAGE'].fields_by_name['role']._loaded_options = None
  _globals['_MESSAGE'].fields_by_name['role']._serialized_options = b'\340A\002'
  _globals['_MESSAGE'].fields_by_name['parts']._loaded_options = None
  _globals['_MESSAGE'].fields_by_name['parts']._serialized_options = b'\340A\002'
  _globals['_ARTIFACT'].fields_by_name['artifact_id']._loaded_options = None
  _globals['_ARTIFACT'].fields_by_name['artifact_id']._serialized_options = b'\340A\002'
  _globals['_ARTIFACT'].fields_by_name['parts']._loaded_options = None
  _globals['_ARTIFACT'].fields_by_name['parts']._serialized_options = b'\340A\002'
  _globals['_TASKSTATUSUPDATEEVENT'].fields_by_name['task_id']._loaded_options = None
  _globals['_TASKSTATUSUPDATEEVENT'].fields_by_name['task_id']._serialized_options = b'\340A\002'
  _globals['_TASKSTATUSUPDATEEVENT'].fields_by_name['context_id']._loaded_options = None
  _globals['_TASKSTATUSUPDATEEVENT'].fields_by_name['context_id']._serialized_options = b'\340A\002'
  _globals['_TASKSTATUSUPDATEEVENT'].fields_by_name['status']._loaded_options = None
  _globals['_TASKSTATUSUPDATEEVENT'].fields_by_name['status']._serialized_options = b'\340A\002'
  _globals['_TASKARTIFACTUPDATEEVENT'].fields_by_name['task_id']._loaded_options = None
  _globals['_TASKARTIFACTUPDATEEVENT'].fields_by_name['task_id']._serialized_options = b'\340A\002'
  _globals['_TASKARTIFACTUPDATEEVENT'].fields_by_name['context_id']._loaded_options = None
  _globals['_TASKARTIFACTUPDATEEVENT'].fields_by_name['context_id']._serialized_options = b'\340A\002'
  _globals['_TASKARTIFACTUPDATEEVENT'].fields_by_name['artifact']._loaded_options = None
  _globals['_TASKARTIFACTUPDATEEVENT'].fields_by_name['artifact']._serialized_options = b'\340A\002'
  _globals['_AUTHENTICATIONINFO'].fields_by_name['scheme']._loaded_options = None
  _globals['_AUTHENTICATIONINFO'].fields_by_name['scheme']._serialized_options = b'\340A\002'
  _globals['_AGENTINTERFACE'].fields_by_name['url']._loaded_options = None
  _globals['_AGENTINTERFACE'].fields_by_name['url']._serialized_options = b'\340A\002'
  _globals['_AGENTINTERFACE'].fields_by_name['protocol_binding']._loaded_options = None
  _globals['_AGENTINTERFACE'].fields_by_name['protocol_binding']._serialized_options = b'\340A\002'
  _globals['_AGENTINTERFACE'].fields_by_name['protocol_version']._loaded_options = None
  _globals['_AGENTINTERFACE'].fields_by_name['protocol_version']._serialized_options = b'\340A\002'
  _globals['_AGENTCARD_SECURITYSCHEMESENTRY']._loaded_options = None
  _globals['_AGENTCARD_SECURITYSCHEMESENTRY']._serialized_options = b'8\001'
  _globals['_AGENTCARD'].fields_by_name['name']._loaded_options = None
  _globals['_AGENTCARD'].fields_by_name['name']._serialized_options = b'\340A\002'
  _globals['_AGENTCARD'].fields_by_name['description']._loaded_options = None
  _globals['_AGENTCARD'].fields_by_name['description']._serialized_options = b'\340A\002'
  _globals['_AGENTCARD'].fields_by_name['supported_interfaces']._loaded_options = None
  _globals['_AGENTCARD'].fields_by_name['supported_interfaces']._serialized_options = b'\340A\002'
  _globals['_AGENTCARD'].fields_by_name['version']._loaded_options = None
  _globals['_AGENTCARD'].fields_by_name['version']._serialized_options = b'\340A\002'
  _globals['_AGENTCARD'].fields_by_name['capabilities']._loaded_options = None
  _globals['_AGENTCARD'].fields_by_name['capabilities']._serialized_options = b'\340A\002'
  _globals['_AGENTCARD'].fields_by_name['default_input_modes']._loaded_options = None
  _globals['_AGENTCARD'].fields_by_name['default_input_modes']._serialized_options = b'\340A\002'
  _globals['_AGENTCARD'].fields_by_name['default_output_modes']._loaded_options = None
  _globals['_AGENTCARD'].fields_by_name['default_output_modes']._serialized_options = b'\340A\002'
  _globals['_AGENTCARD'].fields_by_name['skills']._loaded_options = None
  _globals['_AGENTCARD'].fields_by_name['skills']._serialized_options = b'\340A\002'
  _globals['_AGENTPROVIDER'].fields_by_name['url']._loaded_options = None
  _globals['_AGENTPROVIDER'].fields_by_name['url']._serialized_options = b'\340A\002'
  _globals['_AGENTPROVIDER'].fields_by_name['organization']._loaded_options = None
  _globals['_AGENTPROVIDER'].fields_by_name['organization']._serialized_options = b'\340A\002'
  _globals['_AGENTSKILL'].fields_by_name['id']._loaded_options = None
  _globals['_AGENTSKILL'].fields_by_name['id']._serialized_options = b'\340A\002'
  _globals['_AGENTSKILL'].fields_by_name['name']._loaded_options = None
  _globals['_AGENTSKILL'].fields_by_name['name']._serialized_options = b'\340A\002'
  _globals['_AGENTSKILL'].fields_by_name['description']._loaded_options = None
  _globals['_AGENTSKILL'].fields_by_name['description']._serialized_options = b'\340A\002'
  _globals['_AGENTSKILL'].fields_by_name['tags']._loaded_options = None
  _globals['_AGENTSKILL'].fields_by_name['tags']._serialized_options = b'\340A\002'
  _globals['_AGENTCARDSIGNATURE'].fields_by_name['protected']._loaded_options = None
  _globals['_AGENTCARDSIGNATURE'].fields_by_name['protected']._serialized_options = b'\340A\002'
  _globals['_AGENTCARDSIGNATURE'].fields_by_name['signature']._loaded_options = None
  _globals['_AGENTCARDSIGNATURE'].fields_by_name['signature']._serialized_options = b'\340A\002'
  _globals['_TASKPUSHNOTIFICATIONCONFIG'].fields_by_name['url']._loaded_options = None
  _globals['_TASKPUSHNOTIFICATIONCONFIG'].fields_by_name['url']._serialized_options = b'\340A\002'
  _globals['_SECURITYREQUIREMENT_SCHEMESENTRY']._loaded_options = None
  _globals['_SECURITYREQUIREMENT_SCHEMESENTRY']._serialized_options = b'8\001'
  _globals['_APIKEYSECURITYSCHEME'].fields_by_name['location']._loaded_options = None
  _globals['_APIKEYSECURITYSCHEME'].fields_by_name['location']._serialized_options = b'\340A\002'
  _globals['_APIKEYSECURITYSCHEME'].fields_by_name['name']._loaded_options = None
  _globals['_APIKEYSECURITYSCHEME'].fields_by_name['name']._serialized_options = b'\340A\002'
  _globals['_HTTPAUTHSECURITYSCHEME'].fields_by_name['scheme']._loaded_options = None
  _globals['_HTTPAUTHSECURITYSCHEME'].fields_by_name['scheme']._serialized_options = b'\340A\002'
  _globals['_OAUTH2SECURITYSCHEME'].fields_by_name['flows']._loaded_options = None
  _globals['_OAUTH2SECURITYSCHEME'].fields_by_name['flows']._serialized_options = b'\340A\002'
  _globals['_OPENIDCONNECTSECURITYSCHEME'].fields_by_name['open_id_connect_url']._loaded_options = None
  _globals['_OPENIDCONNECTSECURITYSCHEME'].fields_by_name['open_id_connect_url']._serialized_options = b'\340A\002'
  _globals['_OAUTHFLOWS'].fields_by_name['implicit']._loaded_options = None
  _globals['_OAUTHFLOWS'].fields_by_name['implicit']._serialized_options = b'\030\001'
  _globals['_OAUTHFLOWS'].fields_by_name['password']._loaded_options = None
  _globals['_OAUTHFLOWS'].fields_by_name['password']._serialized_options = b'\030\001'
  _globals['_AUTHORIZATIONCODEOAUTHFLOW_SCOPESENTRY']._loaded_options = None
  _globals['_AUTHORIZATIONCODEOAUTHFLOW_SCOPESENTRY']._serialized_options = b'8\001'
  _globals['_AUTHORIZATIONCODEOAUTHFLOW'].fields_by_name['authorization_url']._loaded_options = None
  _globals['_AUTHORIZATIONCODEOAUTHFLOW'].fields_by_name['authorization_url']._serialized_options = b'\340A\002'
  _globals['_AUTHORIZATIONCODEOAUTHFLOW'].fields_by_name['token_url']._loaded_options = None
  _globals['_AUTHORIZATIONCODEOAUTHFLOW'].fields_by_name['token_url']._serialized_options = b'\340A\002'
  _globals['_AUTHORIZATIONCODEOAUTHFLOW'].fields_by_name['scopes']._loaded_options = None
  _globals['_AUTHORIZATIONCODEOAUTHFLOW'].fields_by_name['scopes']._serialized_options = b'\340A\002'
  _globals['_CLIENTCREDENTIALSOAUTHFLOW_SCOPESENTRY']._loaded_options = None
  _globals['_CLIENTCREDENTIALSOAUTHFLOW_SCOPESENTRY']._serialized_options = b'8\001'
  _globals['_CLIENTCREDENTIALSOAUTHFLOW'].fields_by_name['token_url']._loaded_options = None
  _globals['_CLIENTCREDENTIALSOAUTHFLOW'].fields_by_name['token_url']._serialized_options = b'\340A\002'
  _globals['_CLIENTCREDENTIALSOAUTHFLOW'].fields_by_name['scopes']._loaded_options = None
  _globals['_CLIENTCREDENTIALSOAUTHFLOW'].fields_by_name['scopes']._serialized_options = b'\340A\002'
  _globals['_IMPLICITOAUTHFLOW_SCOPESENTRY']._loaded_options = None
  _globals['_IMPLICITOAUTHFLOW_SCOPESENTRY']._serialized_options = b'8\001'
  _globals['_PASSWORDOAUTHFLOW_SCOPESENTRY']._loaded_options = None
  _globals['_PASSWORDOAUTHFLOW_SCOPESENTRY']._serialized_options = b'8\001'
  _globals['_DEVICECODEOAUTHFLOW_SCOPESENTRY']._loaded_options = None
  _globals['_DEVICECODEOAUTHFLOW_SCOPESENTRY']._serialized_options = b'8\001'
  _globals['_DEVICECODEOAUTHFLOW'].fields_by_name['device_authorization_url']._loaded_options = None
  _globals['_DEVICECODEOAUTHFLOW'].fields_by_name['device_authorization_url']._serialized_options = b'\340A\002'
  _globals['_DEVICECODEOAUTHFLOW'].fields_by_name['token_url']._loaded_options = None
  _globals['_DEVICECODEOAUTHFLOW'].fields_by_name['token_url']._serialized_options = b'\340A\002'
  _globals['_DEVICECODEOAUTHFLOW'].fields_by_name['scopes']._loaded_options = None
  _globals['_DEVICECODEOAUTHFLOW'].fields_by_name['scopes']._serialized_options = b'\340A\002'
  _globals['_SENDMESSAGEREQUEST'].fields_by_name['message']._loaded_options = None
  _globals['_SENDMESSAGEREQUEST'].fields_by_name['message']._serialized_options = b'\340A\002'
  _globals['_GETTASKREQUEST'].fields_by_name['id']._loaded_options = None
  _globals['_GETTASKREQUEST'].fields_by_name['id']._serialized_options = b'\340A\002'
  _globals['_LISTTASKSRESPONSE'].fields_by_name['tasks']._loaded_options = None
  _globals['_LISTTASKSRESPONSE'].fields_by_name['tasks']._serialized_options = b'\340A\002'
  _globals['_LISTTASKSRESPONSE'].fields_by_name['next_page_token']._loaded_options = None
  _globals['_LISTTASKSRESPONSE'].fields_by_name['next_page_token']._serialized_options = b'\340A\002'
  _globals['_LISTTASKSRESPONSE'].fields_by_name['page_size']._loaded_options = None
  _globals['_LISTTASKSRESPONSE'].fields_by_name['page_size']._serialized_options = b'\340A\002'
  _globals['_LISTTASKSRESPONSE'].fields_by_name['total_size']._loaded_options = None
  _globals['_LISTTASKSRESPONSE'].fields_by_name['total_size']._serialized_options = b'\340A\002'
  _globals['_CANCELTASKREQUEST'].fields_by_name['id']._loaded_options = None
  _globals['_CANCELTASKREQUEST'].fields_by_name['id']._serialized_options = b'\340A\002'
  _globals['_GETTASKPUSHNOTIFICATIONCONFIGREQUEST'].fields_by_name['task_id']._loaded_options = None
  _globals['_GETTASKPUSHNOTIFICATIONCONFIGREQUEST'].fields_by_name['task_id']._serialized_options = b'\340A\002'
  _globals['_GETTASKPUSHNOTIFICATIONCONFIGREQUEST'].fields_by_name['id']._loaded_options = None
  _globals['_GETTASKPUSHNOTIFICATIONCONFIGREQUEST'].fields_by_name['id']._serialized_options = b'\340A\002'
  _globals['_DELETETASKPUSHNOTIFICATIONCONFIGREQUEST'].fields_by_name['task_id']._loaded_options = None
  _globals['_DELETETASKPUSHNOTIFICATIONCONFIGREQUEST'].fields_by_name['task_id']._serialized_options = b'\340A\002'
  _globals['_DELETETASKPUSHNOTIFICATIONCONFIGREQUEST'].fields_by_name['id']._loaded_options = None
  _globals['_DELETETASKPUSHNOTIFICATIONCONFIGREQUEST'].fields_by_name['id']._serialized_options = b'\340A\002'
  _globals['_SUBSCRIBETOTASKREQUEST'].fields_by_name['id']._loaded_options = None
  _globals['_SUBSCRIBETOTASKREQUEST'].fields_by_name['id']._serialized_options = b'\340A\002'
  _globals['_LISTTASKPUSHNOTIFICATIONCONFIGSREQUEST'].fields_by_name['task_id']._loaded_options = None
  _globals['_LISTTASKPUSHNOTIFICATIONCONFIGSREQUEST'].fields_by_name['task_id']._serialized_options = b'\340A\002'
  _globals['_A2ASERVICE'].methods_by_name['SendMessage']._loaded_options = None
  _globals['_A2ASERVICE'].methods_by_name['SendMessage']._serialized_options = b'\202\323\344\223\002/\"\r/message:send:\001*Z\033\"\026/{tenant}/message:send:\001*'
  _globals['_A2ASERVICE'].methods_by_name['SendStreamingMessage']._loaded_options = None
  _globals['_A2ASERVICE'].methods_by_name['SendStreamingMessage']._serialized_options = b'\202\323\344\223\0023\"\017/message:stream:\001*Z\035\"\030/{tenant}/message:stream:\001*'
  _globals['_A2ASERVICE'].methods_by_name['GetTask']._loaded_options = None
  _globals['_A2ASERVICE'].methods_by_name['GetTask']._serialized_options = b'\332A\002id\202\323\344\223\002)\022\r/tasks/{id=*}Z\030\022\026/{tenant}/tasks/{id=*}'
  _globals['_A2ASERVICE'].methods_by_name['ListTasks']._loaded_options = None
  _globals['_A2ASERVICE'].methods_by_name['ListTasks']._serialized_options = b'\202\323\344\223\002\033\022\006/tasksZ\021\022\017/{tenant}/tasks'
  _globals['_A2ASERVICE'].methods_by_name['CancelTask']._loaded_options = None
  _globals['_A2ASERVICE'].methods_by_name['CancelTask']._serialized_options = b'\202\323\344\223\002=\"\024/tasks/{id=*}:cancel:\001*Z\"\"\035/{tenant}/tasks/{id=*}:cancel:\001*'
  _globals['_A2ASERVICE'].methods_by_name['SubscribeToTask']._loaded_options = None
  _globals['_A2ASERVICE'].methods_by_name['SubscribeToTask']._serialized_options = b'\202\323\344\223\002=\022\027/tasks/{id=*}:subscribeZ\"\022 /{tenant}/tasks/{id=*}:subscribe'
  _globals['_A2ASERVICE'].methods_by_name['CreateTaskPushNotificationConfig']._loaded_options = None
  _globals['_A2ASERVICE'].methods_by_name['CreateTaskPushNotificationConfig']._serialized_options = b'\332A\016task_id,config\202\323\344\223\002i\"*/tasks/{task_id=*}/pushNotificationConfigs:\001*Z8\"3/{tenant}/tasks/{task_id=*}/pushNotificationConfigs:\001*'
  _globals['_A2ASERVICE'].methods_by_name['GetTaskPushNotificationConfig']._loaded_options = None
  _globals['_A2ASERVICE'].methods_by_name['GetTaskPushNotificationConfig']._serialized_options = b'\332A\ntask_id,id\202\323\344\223\002q\0221/tasks/{task_id=*}/pushNotificationConfigs/{id=*}Z<\022:/{tenant}/tasks/{task_id=*}/pushNotificationConfigs/{id=*}'
  _globals['_A2ASERVICE'].methods_by_name['ListTaskPushNotificationConfigs']._loaded_options = None
  _globals['_A2ASERVICE'].methods_by_name['ListTaskPushNotificationConfigs']._serialized_options = b'\332A\007task_id\202\323\344\223\002c\022*/tasks/{task_id=*}/pushNotificationConfigsZ5\0223/{tenant}/tasks/{task_id=*}/pushNotificationConfigs'
  _globals['_A2ASERVICE'].methods_by_name['GetExtendedAgentCard']._loaded_options = None
  _globals['_A2ASERVICE'].methods_by_name['GetExtendedAgentCard']._serialized_options = b'\202\323\344\223\0023\022\022/extendedAgentCardZ\035\022\033/{tenant}/extendedAgentCard'
  _globals['_A2ASERVICE'].methods_by_name['DeleteTaskPushNotificationConfig']._loaded_options = None
  _globals['_A2ASERVICE'].methods_by_name['DeleteTaskPushNotificationConfig']._serialized_options = b'\332A\ntask_id,id\202\323\344\223\002q*1/tasks/{task_id=*}/pushNotificationConfigs/{id=*}Z<*:/{tenant}/tasks/{task_id=*}/pushNotificationConfigs/{id=*}'
  _globals['_TASKSTATE']._serialized_start=9615
  _globals['_TASKSTATE']._serialized_end=9864
  _globals['_ROLE']._serialized_start=9866
  _globals['_ROLE']._serialized_end=9925
  _globals['_SENDMESSAGECONFIGURATION']._serialized_start=205
  _globals['_SENDMESSAGECONFIGURATION']._serialized_end=499
  _globals['_TASK']._serialized_start=502
  _globals['_TASK']._serialized_end=762
  _globals['_TASKSTATUS']._serialized_start=765
  _globals['_TASKSTATUS']._serialized_end=930
  _globals['_PART']._serialized_start=933
  _globals['_PART']._serialized_end=1170
  _globals['_MESSAGE']._serialized_start=1173
  _globals['_MESSAGE']._serialized_end=1491
  _globals['_ARTIFACT']._serialized_start=1494
  _globals['_ARTIFACT']._serialized_end=1725
  _globals['_TASKSTATUSUPDATEEVENT']._serialized_start=1728
  _globals['_TASKSTATUSUPDATEEVENT']._serialized_end=1922
  _globals['_TASKARTIFACTUPDATEEVENT']._serialized_start=1925
  _globals['_TASKARTIFACTUPDATEEVENT']._serialized_end=2178
  _globals['_AUTHENTICATIONINFO']._serialized_start=2180
  _globals['_AUTHENTICATIONINFO']._serialized_end=2263
  _globals['_AGENTINTERFACE']._serialized_start=2266
  _globals['_AGENTINTERFACE']._serialized_end=2425
  _globals['_AGENTCARD']._serialized_start=2428
  _globals['_AGENTCARD']._serialized_end=3348
  _globals['_AGENTCARD_SECURITYSCHEMESENTRY']._serialized_start=3220
  _globals['_AGENTCARD_SECURITYSCHEMESENTRY']._serialized_end=3313
  _globals['_AGENTPROVIDER']._serialized_start=3350
  _globals['_AGENTPROVIDER']._serialized_end=3429
  _globals['_AGENTCAPABILITIES']._serialized_start=3432
  _globals['_AGENTCAPABILITIES']._serialized_end=3711
  _globals['_AGENTEXTENSION']._serialized_start=3714
  _globals['_AGENTEXTENSION']._serialized_end=3859
  _globals['_AGENTSKILL']._serialized_start=3862
  _globals['_AGENTSKILL']._serialized_end=4165
  _globals['_AGENTCARDSIGNATURE']._serialized_start=4168
  _globals['_AGENTCARDSIGNATURE']._serialized_end=4307
  _globals['_TASKPUSHNOTIFICATIONCONFIG']._serialized_start=4310
  _globals['_TASKPUSHNOTIFICATIONCONFIG']._serialized_end=4519
  _globals['_STRINGLIST']._serialized_start=4521
  _globals['_STRINGLIST']._serialized_end=4553
  _globals['_SECURITYREQUIREMENT']._serialized_start=4556
  _globals['_SECURITYREQUIREMENT']._serialized_end=4731
  _globals['_SECURITYREQUIREMENT_SCHEMESENTRY']._serialized_start=4650
  _globals['_SECURITYREQUIREMENT_SCHEMESENTRY']._serialized_end=4731
  _globals['_SECURITYSCHEME']._serialized_start=4734
  _globals['_SECURITYSCHEME']._serialized_end=5235
  _globals['_APIKEYSECURITYSCHEME']._serialized_start=5237
  _globals['_APIKEYSECURITYSCHEME']._serialized_end=5351
  _globals['_HTTPAUTHSECURITYSCHEME']._serialized_start=5353
  _globals['_HTTPAUTHSECURITYSCHEME']._serialized_end=5477
  _globals['_OAUTH2SECURITYSCHEME']._serialized_start=5480
  _globals['_OAUTH2SECURITYSCHEME']._serialized_end=5634
  _globals['_OPENIDCONNECTSECURITYSCHEME']._serialized_start=5636
  _globals['_OPENIDCONNECTSECURITYSCHEME']._serialized_end=5751
  _globals['_MUTUALTLSSECURITYSCHEME']._serialized_start=5753
  _globals['_MUTUALTLSSECURITYSCHEME']._serialized_end=5812
  _globals['_OAUTHFLOWS']._serialized_start=5815
  _globals['_OAUTHFLOWS']._serialized_end=6206
  _globals['_AUTHORIZATIONCODEOAUTHFLOW']._serialized_start=6209
  _globals['_AUTHORIZATIONCODEOAUTHFLOW']._serialized_end=6530
  _globals['_AUTHORIZATIONCODEOAUTHFLOW_SCOPESENTRY']._serialized_start=6473
  _globals['_AUTHORIZATIONCODEOAUTHFLOW_SCOPESENTRY']._serialized_end=6530
  _globals['_CLIENTCREDENTIALSOAUTHFLOW']._serialized_start=6533
  _globals['_CLIENTCREDENTIALSOAUTHFLOW']._serialized_end=6767
  _globals['_CLIENTCREDENTIALSOAUTHFLOW_SCOPESENTRY']._serialized_start=6473
  _globals['_CLIENTCREDENTIALSOAUTHFLOW_SCOPESENTRY']._serialized_end=6530
  _globals['_IMPLICITOAUTHFLOW']._serialized_start=6770
  _globals['_IMPLICITOAUTHFLOW']._serialized_end=6992
  _globals['_IMPLICITOAUTHFLOW_SCOPESENTRY']._serialized_start=6473
  _globals['_IMPLICITOAUTHFLOW_SCOPESENTRY']._serialized_end=6530
  _globals['_PASSWORDOAUTHFLOW']._serialized_start=6995
  _globals['_PASSWORDOAUTHFLOW']._serialized_end=7201
  _globals['_PASSWORDOAUTHFLOW_SCOPESENTRY']._serialized_start=6473
  _globals['_PASSWORDOAUTHFLOW_SCOPESENTRY']._serialized_end=6530
  _globals['_DEVICECODEOAUTHFLOW']._serialized_start=7204
  _globals['_DEVICECODEOAUTHFLOW']._serialized_end=7487
  _globals['_DEVICECODEOAUTHFLOW_SCOPESENTRY']._serialized_start=6473
  _globals['_DEVICECODEOAUTHFLOW_SCOPESENTRY']._serialized_end=6530
  _globals['_SENDMESSAGEREQUEST']._serialized_start=7490
  _globals['_SENDMESSAGEREQUEST']._serialized_end=7713
  _globals['_GETTASKREQUEST']._serialized_start=7715
  _globals['_GETTASKREQUEST']._serialized_end=7839
  _globals['_LISTTASKSREQUEST']._serialized_start=7842
  _globals['_LISTTASKSREQUEST']._serialized_end=8257
  _globals['_LISTTASKSRESPONSE']._serialized_start=8260
  _globals['_LISTTASKSRESPONSE']._serialized_end=8438
  _globals['_CANCELTASKREQUEST']._serialized_start=8440
  _globals['_CANCELTASKREQUEST']._serialized_end=8557
  _globals['_GETTASKPUSHNOTIFICATIONCONFIGREQUEST']._serialized_start=8559
  _globals['_GETTASKPUSHNOTIFICATIONCONFIGREQUEST']._serialized_end=8672
  _globals['_DELETETASKPUSHNOTIFICATIONCONFIGREQUEST']._serialized_start=8674
  _globals['_DELETETASKPUSHNOTIFICATIONCONFIGREQUEST']._serialized_end=8790
  _globals['_SUBSCRIBETOTASKREQUEST']._serialized_start=8792
  _globals['_SUBSCRIBETOTASKREQUEST']._serialized_end=8861
  _globals['_LISTTASKPUSHNOTIFICATIONCONFIGSREQUEST']._serialized_start=8864
  _globals['_LISTTASKPUSHNOTIFICATIONCONFIGSREQUEST']._serialized_end=9018
  _globals['_GETEXTENDEDAGENTCARDREQUEST']._serialized_start=9020
  _globals['_GETEXTENDEDAGENTCARDREQUEST']._serialized_end=9073
  _globals['_SENDMESSAGERESPONSE']._serialized_start=9075
  _globals['_SENDMESSAGERESPONSE']._serialized_end=9194
  _globals['_STREAMRESPONSE']._serialized_start=9197
  _globals['_STREAMRESPONSE']._serialized_end=9463
  _globals['_LISTTASKPUSHNOTIFICATIONCONFIGSRESPONSE']._serialized_start=9466
  _globals['_LISTTASKPUSHNOTIFICATIONCONFIGSRESPONSE']._serialized_end=9612
  _globals['_A2ASERVICE']._serialized_start=9928
  _globals['_A2ASERVICE']._serialized_end=11871
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

import logging

from typing import TYPE_CHECKING


if TYPE_CHECKING:
    from a2a.server.context import ServerCallContext
from a2a.server.tasks.task_store import TaskStore
from a2a.types.a2a_pb2 import ListTasksRequest, ListTasksResponse, Task


logger = logging.getLogger(__name__)


class CopyingTaskStoreAdapter(TaskStore):
    """An adapter that ensures deep copies of tasks are passed to and returned from the underlying TaskStore.

    This prevents accidental shared mutable state bugs where code modifies a Task object
    retrieved from the store without explicitly saving it, which hides missing save calls.
    """

    def __init__(self, underlying_store: TaskStore):
        self._store = underlying_store

    async def save(self, task: Task, context: ServerCallContext) -> None:
        """Saves a copy of the task to the underlying store."""
        task_copy = Task()
        task_copy.CopyFrom(task)
        await self._store.save(task_copy, context)

    async def get(
        self, task_id: str, context: ServerCallContext
    ) -> Task | None:
        """Retrieves a task from the underlying store and returns a copy."""
        task = await self._store.get(task_id, context)
        if task is None:
            return None
        task_copy = Task()
        task_copy.CopyFrom(task)
        return task_copy

    async def list(
        self,
        params: ListTasksRequest,
        context: ServerCallContext,
    ) -> ListTasksResponse:
        """Retrieves a list of tasks from the underlying store and returns a copy."""
        response = await self._store.list(params, context)
        response_copy = ListTasksResponse()
        response_copy.CopyFrom(response)
        return response_copy

    async def delete(self, task_id: str, context: ServerCallContext) -> None:
        """Deletes a task from the underlying store."""
        await self._store.delete(task_id, context)



import logging

from abc import ABC, abstractmethod

from a2a.server.context import ServerCallContext
from a2a.types.a2a_pb2 import TaskPushNotificationConfig


logger = logging.getLogger(__name__)


class PushNotificationConfigStore(ABC):
    """Interface for storing and retrieving push notification configurations for tasks."""

    @abstractmethod
    async def set_info(
        self,
        task_id: str,
        notification_config: TaskPushNotificationConfig,
        context: ServerCallContext,
    ) -> None:
        """Sets or updates the push notification configuration for a task."""

    @abstractmethod
    async def get_info(
        self,
        task_id: str,
        context: ServerCallContext,
    ) -> list[TaskPushNotificationConfig]:
        """Retrieves push notification configurations for a task, scoped to the caller.

        This is the user-callable read path. Implementations MUST return
        only configurations owned by the caller (as resolved from
        context).
        """

    async def get_info_for_dispatch(
        self,
        task_id: str,
    ) -> list[TaskPushNotificationConfig]:
        """Retrieves all push notification configurations for a task, across all owners.

        This is the internal read path used by the push-notification
        dispatch loop. Implementations SHOULD override this method to
        return every configuration registered for task_id regardless of
        which user registered it. Authorization already happened at
        registration time and the dispatch path fires every registered
        webhook for the task.

        The default implementation falls back to calling get_info with
        a synthetic empty ServerCallContext. This preserves 1.0
        behavior for subclasses that have not implemented the override
        but is INCORRECT for any deployment with multiple owners: the
        empty context resolves to the empty-string owner partition and
        returns no configs (silently dropping every notification). A
        warning is logged on every call to flag the misconfiguration.
        Custom subclasses MUST override this method to deliver
        notifications correctly in multi-owner deployments.
        """
        logger.warning(
            '%s does not override '
            'PushNotificationConfigStore.get_info_for_dispatch; falling back '
            'to a context-less get_info call which silently drops '
            'notifications in any deployment with multiple owners. Override '
            'get_info_for_dispatch to return all configs for task_id across '
            'every owner.',
            type(self).__name__,
        )
        return await self.get_info(task_id, ServerCallContext())

    @abstractmethod
    async def delete_info(
        self,
        task_id: str,
        context: ServerCallContext,
        config_id: str | None = None,
    ) -> None:
        """Deletes the push notification configuration for a task."""



import functools
import inspect
import logging

from abc import ABC, abstractmethod
from collections.abc import AsyncGenerator, Callable
from typing import Any

from google.protobuf.message import Message as ProtoMessage

from a2a.server.context import ServerCallContext
from a2a.server.events.event_queue import Event
from a2a.types.a2a_pb2 import (
    AgentCard,
    CancelTaskRequest,
    DeleteTaskPushNotificationConfigRequest,
    GetExtendedAgentCardRequest,
    GetTaskPushNotificationConfigRequest,
    GetTaskRequest,
    ListTaskPushNotificationConfigsRequest,
    ListTaskPushNotificationConfigsResponse,
    ListTasksRequest,
    ListTasksResponse,
    Message,
    SendMessageRequest,
    SubscribeToTaskRequest,
    Task,
    TaskPushNotificationConfig,
)
from a2a.utils.errors import UnsupportedOperationError
from a2a.utils.proto_utils import validate_proto_required_fields


class RequestHandler(ABC):
    """A2A request handler interface.

    This interface defines the methods that an A2A server implementation must
    provide to handle incoming A2A requests from any transport (gRPC, REST, JSON-RPC).
    """

    @abstractmethod
    async def on_get_task(
        self,
        params: GetTaskRequest,
        context: ServerCallContext,
    ) -> Task | None:
        """Handles the 'tasks/get' method.

        Retrieves the state and history of a specific task.

        Args:
            params: Parameters specifying the task ID and optionally history length.
            context: Context provided by the server.

        Returns:
            The `Task` object if found, otherwise `None`.
        """

    @abstractmethod
    async def on_list_tasks(
        self, params: ListTasksRequest, context: ServerCallContext
    ) -> ListTasksResponse:
        """Handles the tasks/list method.

        Retrieves all tasks for an agent. Supports filtering, pagination,
        ordering, limiting the history length, excluding artifacts, etc.

        Args:
            params: Parameters with filtering criteria.
            context: Context provided by the server.

        Returns:
            The `ListTasksResponse` containing the tasks.
        """

    @abstractmethod
    async def on_cancel_task(
        self,
        params: CancelTaskRequest,
        context: ServerCallContext,
    ) -> Task | None:
        """Handles the 'tasks/cancel' method.

        Requests the agent to cancel an ongoing task.

        Args:
            params: Parameters specifying the task ID.
            context: Context provided by the server.

        Returns:
            The `Task` object with its status updated to canceled, or `None` if the task was not found.
        """

    @abstractmethod
    async def on_message_send(
        self,
        params: SendMessageRequest,
        context: ServerCallContext,
    ) -> Task | Message:
        """Handles the 'message/send' method (non-streaming).

        Sends a message to the agent to create, continue, or restart a task,
        and waits for the final result (Task or Message).

        Args:
            params: Parameters including the message and configuration.
            context: Context provided by the server.

        Returns:
            The final `Task` object or a final `Message` object.
        """

    @abstractmethod
    async def on_message_send_stream(
        self,
        params: SendMessageRequest,
        context: ServerCallContext,
    ) -> AsyncGenerator[Event]:
        """Handles the 'message/stream' method (streaming).

        Sends a message to the agent and yields stream events as they are
        produced (Task updates, Message chunks, Artifact updates).

        Args:
            params: Parameters including the message and configuration.
            context: Context provided by the server.

        Yields:
            `Event` objects from the agent's execution.
        """
        # This is needed for typechecker to recognise this method as an async generator.
        raise UnsupportedOperationError
        yield

    @abstractmethod
    async def on_create_task_push_notification_config(
        self,
        params: TaskPushNotificationConfig,
        context: ServerCallContext,
    ) -> TaskPushNotificationConfig:
        """Handles the 'tasks/pushNotificationConfig/create' method.

        Sets or updates the push notification configuration for a task.

        Args:
            params: Parameters including the task ID and push notification configuration.
            context: Context provided by the server.

        Returns:
            The provided `TaskPushNotificationConfig` upon success.
        """

    @abstractmethod
    async def on_get_task_push_notification_config(
        self,
        params: GetTaskPushNotificationConfigRequest,
        context: ServerCallContext,
    ) -> TaskPushNotificationConfig:
        """Handles the 'tasks/pushNotificationConfig/get' method.

        Retrieves the current push notification configuration for a task.

        Args:
            params: Parameters including the task ID.
            context: Context provided by the server.

        Returns:
            The `TaskPushNotificationConfig` for the task.
        """

    @abstractmethod
    async def on_subscribe_to_task(
        self,
        params: SubscribeToTaskRequest,
        context: ServerCallContext,
    ) -> AsyncGenerator[Event]:
        """Handles the 'SubscribeToTask' method.

        Allows a client to subscribe to a running streaming task's event stream.

        Args:
            params: Parameters including the task ID.
            context: Context provided by the server.

        Yields:
             `Event` objects from the agent's ongoing execution for the specified task.
        """
        raise UnsupportedOperationError
        yield

    @abstractmethod
    async def on_list_task_push_notification_configs(
        self,
        params: ListTaskPushNotificationConfigsRequest,
        context: ServerCallContext,
    ) -> ListTaskPushNotificationConfigsResponse:
        """Handles the 'ListTaskPushNotificationConfigs' method.

        Retrieves the current push notification configurations for a task.

        Args:
            params: Parameters including the task ID.
            context: Context provided by the server.

        Returns:
            The `list[TaskPushNotificationConfig]` for the task.
        """

    @abstractmethod
    async def on_delete_task_push_notification_config(
        self,
        params: DeleteTaskPushNotificationConfigRequest,
        context: ServerCallContext,
    ) -> None:
        """Handles the 'tasks/pushNotificationConfig/delete' method.

        Deletes a push notification configuration associated with a task.

        Args:
            params: Parameters including the task ID.
            context: Context provided by the server.

        Returns:
            None
        """

    @abstractmethod
    async def on_get_extended_agent_card(
        self,
        params: GetExtendedAgentCardRequest,
        context: ServerCallContext,
    ) -> AgentCard:
        """Handles the 'GetExtendedAgentCard' method.

        Retrieves the extended agent card for the agent.

        Args:
            params: Parameters for the request.
            context: Context provided by the server.

        Returns:
            The `AgentCard` object representing the extended properties of the agent.

        """


def validate_request_params(method: Callable) -> Callable:
    """Decorator for RequestHandler methods to validate required fields on incoming requests."""
    if inspect.isasyncgenfunction(method):

        @functools.wraps(method)
        async def async_gen_wrapper(
            self: RequestHandler,
            params: ProtoMessage,
            context: ServerCallContext,
            *args: Any,
            **kwargs: Any,
        ) -> Any:
            if params is not None:
                validate_proto_required_fields(params)
            # Ensure the inner async generator is closed explicitly;
            # bare async-for does not call aclose() on GeneratorExit,
            # which on Python 3.12+ prevents the except/finally blocks
            # in on_message_send_stream from running on client disconnect
            # (background_consume and cleanup_producer tasks are never created).
            inner = method(self, params, context, *args, **kwargs)
            try:
                async for item in inner:
                    yield item
            finally:
                await inner.aclose()

        return async_gen_wrapper

    @functools.wraps(method)
    async def async_wrapper(
        self: RequestHandler,
        params: ProtoMessage,
        context: ServerCallContext,
        *args: Any,
        **kwargs: Any,
    ) -> Any:
        if params is not None:
            validate_proto_required_fields(params)
        return await method(self, params, context, *args, **kwargs)

    return async_wrapper


def validate(
    expression: Callable[[Any], bool],
    error_message: str | None = None,
    error_type: type[Exception] = UnsupportedOperationError,
) -> Callable:
    """Decorator that validates if a given expression evaluates to True.

    Typically used on class methods to check capabilities or configuration
    before executing the method's logic. If the expression is False,
    the specified `error_type` (defaults to `UnsupportedOperationError`) is raised.

    Args:
        expression: A callable that takes the instance (`self`) as its argument
                    and returns a boolean.
        error_message: An optional custom error message for the error raised.
                       If None, the string representation of the expression will be used.
        error_type: The exception class to raise on validation failure.
                   Must take a `message` keyword argument (inherited from A2AError).

    Examples:
        Demonstrating with an async method:
        >>> import asyncio
        >>> from a2a.utils.errors import UnsupportedOperationError
        >>>
        >>> class MyAgent:
        ...     def __init__(self, streaming_enabled: bool):
        ...         self.streaming_enabled = streaming_enabled
        ...
        ...     @validate(
        ...         lambda self: self.streaming_enabled,
        ...         'Streaming is not enabled for this agent',
        ...     )
        ...     async def stream_response(self, message: str):
        ...         return f'Streaming: {message}'
        >>>
        >>> async def run_async_test():
        ...     # Successful call
        ...     agent_ok = MyAgent(streaming_enabled=True)
        ...     result = await agent_ok.stream_response('hello')
        ...     print(result)
        ...
        ...     # Call that fails validation
        ...     agent_fail = MyAgent(streaming_enabled=False)
        ...     try:
        ...         await agent_fail.stream_response('world')
        ...     except UnsupportedOperationError as e:
        ...         print(e.message)
        >>>
        >>> asyncio.run(run_async_test())
        Streaming: hello
        Streaming is not enabled for this agent

        Demonstrating with a sync method:
        >>> class SecureAgent:
        ...     def __init__(self):
        ...         self.auth_enabled = False
        ...
        ...     @validate(
        ...         lambda self: self.auth_enabled,
        ...         'Authentication must be enabled for this operation',
        ...     )
        ...     def secure_operation(self, data: str):
        ...         return f'Processing secure data: {data}'
        >>>
        >>> # Error case example
        >>> agent = SecureAgent()
        >>> try:
        ...     agent.secure_operation('secret')
        ... except UnsupportedOperationError as e:
        ...     print(e.message)
        Authentication must be enabled for this operation

    Note:
        This decorator works with both sync and async methods automatically.
    """

    def decorator(function: Callable) -> Callable:
        if inspect.isasyncgenfunction(function):

            @functools.wraps(function)
            async def async_gen_wrapper(self: Any, *args, **kwargs) -> Any:
                if not expression(self):
                    final_message = error_message or str(expression)
                    logging.getLogger(__name__).error(
                        'Validation failure: %s', final_message
                    )
                    raise error_type(final_message)
                inner = function(self, *args, **kwargs)
                try:
                    async for item in inner:
                        yield item
                finally:
                    await inner.aclose()

            return async_gen_wrapper

        if inspect.iscoroutinefunction(function):

            @functools.wraps(function)
            async def async_wrapper(self: Any, *args, **kwargs) -> Any:
                if not expression(self):
                    final_message = error_message or str(expression)
                    logging.getLogger(__name__).error(
                        'Validation failure: %s', final_message
                    )
                    raise error_type(final_message)
                return await function(self, *args, **kwargs)

            return async_wrapper

        @functools.wraps(function)
        def sync_wrapper(self: Any, *args, **kwargs) -> Any:
            if not expression(self):
                final_message = error_message or str(expression)
                logging.getLogger(__name__).error(
                    'Validation failure: %s', final_message
                )
                raise error_type(final_message)
            return function(self, *args, **kwargs)

        return sync_wrapper

    return decorator
